@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -46,33 +46,31 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.PrintWriter;
 import java.net.URL;
+import java.time.format.TextStyle;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Locale;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
 
-import javax.imageio.ImageIO;
 import javax.swing.UIManager;
 
-import net.sourceforge.plantuml.activitydiagram.ActivityDiagramFactory;
-import net.sourceforge.plantuml.classdiagram.ClassDiagramFactory;
+import net.sourceforge.plantuml.code.NoPlantumlCompressionException;
 import net.sourceforge.plantuml.code.Transcoder;
 import net.sourceforge.plantuml.code.TranscoderUtil;
-import net.sourceforge.plantuml.command.UmlDiagramFactory;
-import net.sourceforge.plantuml.descdiagram.DescriptionDiagramFactory;
 import net.sourceforge.plantuml.ftp.FtpServer;
-import net.sourceforge.plantuml.objectdiagram.ObjectDiagramFactory;
+import net.sourceforge.plantuml.picoweb.PicoWebServer;
 import net.sourceforge.plantuml.png.MetadataTag;
 import net.sourceforge.plantuml.preproc.Stdlib;
-import net.sourceforge.plantuml.sequencediagram.SequenceDiagramFactory;
-import net.sourceforge.plantuml.statediagram.StateDiagramFactory;
+import net.sourceforge.plantuml.security.ImageIO;
+import net.sourceforge.plantuml.security.SFile;
+import net.sourceforge.plantuml.security.SecurityUtils;
+import net.sourceforge.plantuml.sprite.SpriteGrayLevel;
+import net.sourceforge.plantuml.sprite.SpriteUtils;
 import net.sourceforge.plantuml.stats.StatsUtils;
-import net.sourceforge.plantuml.swing.MainWindow2;
+import net.sourceforge.plantuml.swing.MainWindow;
 import net.sourceforge.plantuml.syntax.LanguageDescriptor;
-import net.sourceforge.plantuml.ugraphic.sprite.SpriteGrayLevel;
-import net.sourceforge.plantuml.ugraphic.sprite.SpriteUtils;
 import net.sourceforge.plantuml.utils.Cypher;
 import net.sourceforge.plantuml.version.Version;
 
@@ -80,12 +78,17 @@ public class Run {
 
 	private static Cypher cypher;
 
-	public static void main(String[] argsArray) throws IOException, InterruptedException {
+	public static void main(String[] argsArray)
+			throws NoPlantumlCompressionException, IOException, InterruptedException {
 		System.setProperty("log4j.debug", "false");
 		final long start = System.currentTimeMillis();
 		if (argsArray.length > 0 && argsArray[0].equalsIgnoreCase("-headless")) {
 			System.setProperty("java.awt.headless", "true");
 		}
+		if (argsArray.length > 0 && argsArray[0].equalsIgnoreCase("--de")) {
+			debugGantt();
+			return;
+		}
 		saveCommandLine(argsArray);
 		final Option option = new Option(argsArray);
 		ProgressBar.setEnable(option.isTextProgressBar());
@@ -121,6 +124,7 @@ public class Run {
 			encodeSprite(option.getResult());
 			return;
 		}
+		Log.info("SecurityProfile " + SecurityUtils.getSecurityProfile());
 		if (OptionFlags.getInstance().isVerbose()) {
 			Log.info("PlantUML Version " + Version.versionString());
 			Log.info("GraphicsEnvironment.isHeadless() " + GraphicsEnvironment.isHeadless());
@@ -141,15 +145,18 @@ public class Run {
 			return;
 		}
 
+		if (option.getPicowebPort() != -1) {
+			goPicoweb(option);
+			return;
+		}
+
 		forceOpenJdkResourceLoad();
 		if (option.getPreprocessorOutputMode() == OptionPreprocOutputMode.CYPHER) {
 			cypher = new LanguageDescriptor().getCypher();
 		}
-		boolean error = false;
+		final ErrorStatus error = ErrorStatus.init();
 		boolean forceQuit = false;
-		if (option.isPattern()) {
-			managePattern();
-		} else if (OptionFlags.getInstance().isGui()) {
+		if (OptionFlags.getInstance().isGui()) {
 			try {
 				UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
 			} catch (Exception e) {
@@ -162,9 +169,17 @@ public class Run {
 					dir = f;
 				}
 			}
-			new MainWindow2(option, dir);
+			try {
+				new MainWindow(option, dir);
+			} catch (java.awt.HeadlessException e) {
+				System.err.println("There is an issue with your server. You will find some tips here:");
+				System.err.println("https://forum.plantuml.net/3399/problem-with-x11-and-headless-exception");
+				System.err.println("https://plantuml.com/en/faq#239d64f675c3e515");
+				throw e;
+			}
+
 		} else if (option.isPipe() || option.isPipeMap() || option.isSyntax()) {
-			error = managePipe(option);
+			managePipe(option, error);
 			forceQuit = true;
 		} else if (option.isFailfast2()) {
 			if (option.isSplash()) {
@@ -172,21 +187,21 @@ public class Run {
 			}
 			final long start2 = System.currentTimeMillis();
 			option.setCheckOnly(true);
-			error = manageAllFiles(option);
+			manageAllFiles(option, error);
 			option.setCheckOnly(false);
 			if (option.isDuration()) {
 				final double duration = (System.currentTimeMillis() - start2) / 1000.0;
 				Log.error("Check Duration = " + duration + " seconds");
 			}
-			if (error == false) {
-				error = manageAllFiles(option);
+			if (error.hasError() == false) {
+				manageAllFiles(option, error);
 			}
 			forceQuit = true;
 		} else {
 			if (option.isSplash()) {
 				Splash.createSplash();
 			}
-			error = manageAllFiles(option);
+			manageAllFiles(option, error);
 			forceQuit = true;
 		}
 
@@ -195,13 +210,17 @@ public class Run {
 			Log.error("Duration = " + duration + " seconds");
 		}
 
-		if (error) {
-			Log.error("Some diagram description contains errors");
-			System.exit(1);
-		}
+		if (OptionFlags.getInstance().isGui() == false) {
+			if (error.hasError() || error.isNoData()) {
+				option.getStdrpt().finalMessage(error);
+			}
+			if (error.hasError()) {
+				System.exit(error.getExitCode());
+			}
 
-		if (forceQuit && OptionFlags.getInstance().isSystemExit()) {
-			System.exit(0);
+			if (forceQuit && OptionFlags.getInstance().isSystemExit()) {
+				System.exit(0);
+			}
 		}
 	}
 
@@ -265,15 +284,19 @@ public class Run {
 		final URL source;
 		final String lowerPath = StringUtils.goLowerCase(path);
 		if (lowerPath.startsWith(httpProtocol) || lowerPath.startsWith(httpsProtocol)) {
-			source = new URL(path);
+			source = new java.net.URL(path);
 			final String p = source.getPath();
 			fileName = p.substring(p.lastIndexOf('/') + 1, p.length());
 		} else {
-			final File f = new File(path);
+			final SFile f = new SFile(path);
 			source = f.toURI().toURL();
 			fileName = f.getName();
 		}
 
+		if (source == null) {
+			return;
+		}
+
 		InputStream stream = null;
 		final BufferedImage im;
 		try {
@@ -286,8 +309,8 @@ public class Run {
 		}
 
 		final String name = getSpriteName(fileName);
-		final String s = compressed ? SpriteUtils.encodeCompressed(im, name, level) : SpriteUtils.encode(im, name,
-				level);
+		final String s = compressed ? SpriteUtils.encodeCompressed(im, name, level)
+				: SpriteUtils.encode(im, name, level);
 		System.out.println(s);
 	}
 
@@ -318,11 +341,15 @@ public class Run {
 		ftpServer.go();
 	}
 
+	private static void goPicoweb(Option option) throws IOException {
+		PicoWebServer.startServer(option.getPicowebPort(), option.getPicowebBindAddress());
+	}
+
 	public static void printFonts() {
 		final Font fonts[] = GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts();
 		for (Font f : fonts) {
-			System.out.println("f=" + f + "/" + f.getPSName() + "/" + f.getName() + "/" + f.getFontName() + "/"
-					+ f.getFamily());
+			System.out.println(
+					"f=" + f + "/" + f.getPSName() + "/" + f.getName() + "/" + f.getFontName() + "/" + f.getFamily());
 		}
 		final String name[] = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
 		for (String n : name) {
@@ -330,41 +357,23 @@ public class Run {
 		}
 	}
 
-	private static void managePattern() {
-		printPattern(new SequenceDiagramFactory());
-		printPattern(new ClassDiagramFactory());
-		printPattern(new ActivityDiagramFactory());
-		printPattern(new DescriptionDiagramFactory());
-		// printPattern(new ComponentDiagramFactory());
-		printPattern(new StateDiagramFactory());
-		printPattern(new ObjectDiagramFactory());
-	}
-
-	private static void printPattern(UmlDiagramFactory factory) {
-		System.out.println();
-		System.out.println(factory.getClass().getSimpleName().replaceAll("Factory", ""));
-		final List<String> descriptions = factory.getDescription();
-		for (String s : descriptions) {
-			System.out.println(s);
-		}
-	}
-
-	private static boolean managePipe(Option option) throws IOException {
+	private static void managePipe(Option option, ErrorStatus error) throws IOException {
 		final String charset = option.getCharset();
-		return new Pipe(option, System.out, System.in, charset).managePipe();
+		new Pipe(option, System.out, System.in, charset).managePipe(error);
 	}
 
-	private static boolean manageAllFiles(Option option) throws IOException, InterruptedException {
+	private static void manageAllFiles(Option option, ErrorStatus error)
+			throws NoPlantumlCompressionException, InterruptedException {
 
-		File lockFile = null;
+		SFile lockFile = null;
 		try {
 			if (OptionFlags.getInstance().isWord()) {
-				final File dir = new File(option.getResult().get(0));
-				final File javaIsRunningFile = new File(dir, "javaisrunning.tmp");
+				final SFile dir = new SFile(option.getResult().get(0));
+				final SFile javaIsRunningFile = dir.file("javaisrunning.tmp");
 				javaIsRunningFile.delete();
-				lockFile = new File(dir, "javaumllock.tmp");
+				lockFile = dir.file("javaumllock.tmp");
 			}
-			return processArgs(option);
+			processArgs(option, error);
 		} finally {
 			if (lockFile != null) {
 				lockFile.delete();
@@ -373,15 +382,17 @@ public class Run {
 
 	}
 
-	private static boolean processArgs(Option option) throws IOException, InterruptedException {
+	private static void processArgs(Option option, ErrorStatus error)
+			throws NoPlantumlCompressionException, InterruptedException {
 		if (option.isDecodeurl() == false && option.getNbThreads() > 1 && option.isCheckOnly() == false
 				&& OptionFlags.getInstance().isExtractFromMetadata() == false) {
-			return multithread(option);
+			multithread(option, error);
+			return;
 		}
-		boolean errorGlobal = false;
-		final List<File> files = new ArrayList<File>();
+		final List<File> files = new ArrayList<>();
 		for (String s : option.getResult()) {
 			if (option.isDecodeurl()) {
+				error.goOk();
 				final Transcoder transcoder = TranscoderUtil.getDefaultTranscoder();
 				System.out.println("@startuml");
 				System.out.println(transcoder.decode(s));
@@ -392,55 +403,54 @@ public class Run {
 				files.addAll(group.getFiles());
 			}
 		}
-		Log.info("Found " + files.size() + " files");
+		foundNbFiles(files.size());
 		for (File f : files) {
 			try {
-				final boolean error = manageFileInternal(f, option);
-				if (error) {
-					errorGlobal = true;
-				}
-				incDone(error);
-				if (error && option.isFailfastOrFailfast2()) {
-					return true;
+				manageFileInternal(f, option, error);
+				incDone(error.hasError());
+				if (error.hasError() && option.isFailfastOrFailfast2()) {
+					return;
 				}
 			} catch (IOException e) {
 				e.printStackTrace();
 			}
 		}
-		return errorGlobal;
 	}
 
-	private static boolean multithread(final Option option) throws InterruptedException {
+	private static void multithread(final Option option, final ErrorStatus error) throws InterruptedException {
 		Log.info("Using several threads: " + option.getNbThreads());
 		final ExecutorService executor = Executors.newFixedThreadPool(option.getNbThreads());
-		final AtomicBoolean errors = new AtomicBoolean(false);
+
+		int nb = 0;
 		for (String s : option.getResult()) {
 			final FileGroup group = new FileGroup(s, option.getExcludes(), option);
 			for (final File f : group.getFiles()) {
 				incTotal(1);
+				nb++;
 				executor.submit(new Runnable() {
 					public void run() {
-						if (errors.get() && option.isFailfastOrFailfast2()) {
+						if (error.hasError() && option.isFailfastOrFailfast2()) {
 							return;
 						}
 						try {
-							final boolean error = manageFileInternal(f, option);
-							if (error) {
-								errors.set(true);
-							}
+							manageFileInternal(f, option, error);
 						} catch (IOException e) {
 							e.printStackTrace();
 						} catch (InterruptedException e) {
 							e.printStackTrace();
 						}
-						incDone(errors.get());
+						incDone(error.hasError());
 					}
 				});
 			}
 		}
+		foundNbFiles(nb);
 		executor.shutdown();
 		executor.awaitTermination(Long.MAX_VALUE, TimeUnit.SECONDS);
-		return errors.get();
+	}
+
+	private static void foundNbFiles(int nb) {
+		Log.info("Found " + nb + " files");
 	}
 
 	private static void incDone(boolean error) {
@@ -453,46 +463,79 @@ public class Run {
 		ProgressBar.incTotal(nb);
 	}
 
-	private static boolean manageFileInternal(File f, Option option) throws IOException, InterruptedException {
-		Log.info("Working on " + f.getAbsolutePath());
+	private static void manageFileInternal(File f, Option option, ErrorStatus error)
+			throws IOException, InterruptedException {
+		Log.info("Working on " + f.getPath());
 		if (OptionFlags.getInstance().isExtractFromMetadata()) {
 			System.out.println("------------------------");
 			System.out.println(f);
 			// new Metadata().readAndDisplayMetadata(f);
 			System.out.println();
-			System.out.println(new MetadataTag(f, "plantuml").getData());
+			error.goOk();
+			final String data = new MetadataTag(f, "plantuml").getData();
+			// File file = SecurityUtils.File("tmp.txt");
+			// PrintWriter pw = SecurityUtils.PrintWriter(file, "UTF-8");
+			// pw.println(NastyEncoder.fromISO_8859_1(data));
+			// pw.close();
+
+			System.out.println(data);
 			System.out.println("------------------------");
-			return false;
+			return;
 		}
 		final ISourceFileReader sourceFileReader;
 		if (option.getOutputFile() == null) {
-			sourceFileReader = new SourceFileReader(option.getDefaultDefines(f), f, option.getOutputDir(),
-					option.getConfig(), option.getCharset(), option.getFileFormatOption());
+			File outputDir = option.getOutputDir();
+			if (outputDir != null && outputDir.getPath().endsWith("$")) {
+				final String path = outputDir.getPath();
+				outputDir = new File(path.substring(0, path.length() - 1)).getAbsoluteFile();
+				sourceFileReader = new SourceFileReaderCopyCat(option.getDefaultDefines(f), f, outputDir,
+						option.getConfig(), option.getCharset(), option.getFileFormatOption());
+			} else {
+				sourceFileReader = new SourceFileReader(option.getDefaultDefines(f), f, outputDir, option.getConfig(),
+						option.getCharset(), option.getFileFormatOption());
+			}
 		} else {
-			sourceFileReader = new SourceFileReader2(option.getDefaultDefines(f), f, option.getOutputFile(),
+			sourceFileReader = new SourceFileReaderHardFile(option.getDefaultDefines(f), f, option.getOutputFile(),
 					option.getConfig(), option.getCharset(), option.getFileFormatOption());
 		}
 		sourceFileReader.setCheckMetadata(option.isCheckMetadata());
+		((SourceFileReaderAbstract) sourceFileReader).setNoerror(option.isNoerror());
 
 		if (option.isComputeurl()) {
+			error.goOk();
 			for (BlockUml s : sourceFileReader.getBlocks()) {
 				System.out.println(s.getEncodedUrl());
 			}
-			return false;
+			return;
 		}
 		if (option.isCheckOnly()) {
+			error.goOk();
 			final boolean hasError = sourceFileReader.hasError();
-			final List<GeneratedImage> result = sourceFileReader.getGeneratedImages();
-			hasErrors(f, result);
-			return hasError;
+			if (hasError) {
+				error.goWithError();
+			}
+			// final List<GeneratedImage> result = sourceFileReader.getGeneratedImages();
+			// hasErrors(f, result, error);
+			return;
 		}
 		if (option.getPreprocessorOutputMode() != null) {
 			extractPreproc(option, sourceFileReader);
-			return false;
+			error.goOk();
+			return;
 		}
 		final List<GeneratedImage> result = sourceFileReader.getGeneratedImages();
+		final Stdrpt rpt = option.getStdrpt();
+		if (result.size() == 0) {
+			Log.error("Warning: no image in " + f.getPath());
+			rpt.printInfo(System.err, null);
+			// error.goNoData();
+			return;
+		}
+		for (BlockUml s : sourceFileReader.getBlocks()) {
+			rpt.printInfo(System.err, s.getDiagram());
+		}
 
-		return hasErrors(f, result);
+		hasErrors(f, result, error, rpt);
 	}
 
 	private static void extractPreproc(Option option, final ISourceFileReader sourceFileReader) throws IOException {
@@ -500,12 +543,22 @@ public class Run {
 		for (BlockUml blockUml : sourceFileReader.getBlocks()) {
 			final SuggestedFile suggested = ((SourceFileReaderAbstract) sourceFileReader).getSuggestedFile(blockUml)
 					.withPreprocFormat();
-			final File file = suggested.getFile(0);
-			Log.info("Export preprocessing source to " + file.getAbsolutePath());
-			final PrintWriter pw = charset == null ? new PrintWriter(file) : new PrintWriter(file, charset);
-			for (CharSequence s : blockUml.getDefinition(true)) {
+			final SFile file = suggested.getFile(0);
+			Log.info("Export preprocessing source to " + file.getPrintablePath());
+			final PrintWriter pw = charset == null ? file.createPrintWriter() : file.createPrintWriter(charset);
+			int level = 0;
+			for (CharSequence cs : blockUml.getDefinition(true)) {
+				String s = cs.toString();
 				if (cypher != null) {
-					s = cypher.cypher(s.toString());
+					if (s.contains("skinparam") && s.contains("{")) {
+						level++;
+					}
+					if (level == 0 && s.contains("skinparam") == false) {
+						s = cypher.cypher(s);
+					}
+					if (level > 0 && s.contains("}")) {
+						level--;
+					}
 				}
 				pw.println(s);
 			}
@@ -513,16 +566,33 @@ public class Run {
 		}
 	}
 
-	private static boolean hasErrors(File f, final List<GeneratedImage> list) throws IOException {
-		boolean result = false;
-		for (GeneratedImage i : list) {
-			final int lineError = i.lineErrorRaw();
+	private static void hasErrors(File file, final List<GeneratedImage> list, ErrorStatus error, Stdrpt stdrpt)
+			throws IOException {
+		if (list.size() == 0) {
+			// error.goNoData();
+			return;
+		}
+		for (GeneratedImage image : list) {
+			final int lineError = image.lineErrorRaw();
 			if (lineError != -1) {
-				Log.error("Error line " + lineError + " in file: " + f.getCanonicalPath());
-				result = true;
+				stdrpt.errorLine(lineError, file);
+				error.goWithError();
+				return;
+			}
+		}
+		error.goOk();
+	}
+
+	public static void debugGantt() {
+		final Locale locale = Locale.GERMAN;
+		for (java.time.Month month : java.time.Month.values()) {
+			System.err.println("Testing locale " + locale + " " + month);
+			for (TextStyle style : TextStyle.values()) {
+				final String s = month.getDisplayName(style, locale);
+				System.err.println(style + " --> '" + s + "'");
+
 			}
 		}
-		return result;
 	}
 
 }
