@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -35,34 +35,34 @@
  */
 package net.sourceforge.plantuml.activitydiagram3;
 
-import java.awt.geom.Dimension2D;
 import java.io.IOException;
 import java.io.OutputStream;
+import java.util.Objects;
 
-import net.sourceforge.plantuml.AnnotatedWorker;
-import net.sourceforge.plantuml.Dimension2DDouble;
 import net.sourceforge.plantuml.FileFormatOption;
-import net.sourceforge.plantuml.ISkinParam;
-import net.sourceforge.plantuml.Scale;
+import net.sourceforge.plantuml.ISkinSimple;
 import net.sourceforge.plantuml.UmlDiagram;
 import net.sourceforge.plantuml.UmlDiagramType;
 import net.sourceforge.plantuml.Url;
 import net.sourceforge.plantuml.activitydiagram3.ftile.BoxStyle;
-import net.sourceforge.plantuml.activitydiagram3.ftile.SwimlanesC;
+import net.sourceforge.plantuml.activitydiagram3.ftile.Swimlanes;
 import net.sourceforge.plantuml.command.CommandExecutionResult;
 import net.sourceforge.plantuml.core.DiagramDescription;
 import net.sourceforge.plantuml.core.ImageData;
+import net.sourceforge.plantuml.core.UmlSource;
 import net.sourceforge.plantuml.cucadiagram.Display;
-import net.sourceforge.plantuml.graphic.HtmlColor;
+import net.sourceforge.plantuml.cucadiagram.Stereotype;
 import net.sourceforge.plantuml.graphic.Rainbow;
+import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.graphic.TextBlockRecentred;
+import net.sourceforge.plantuml.graphic.USymbol;
 import net.sourceforge.plantuml.graphic.color.Colors;
 import net.sourceforge.plantuml.sequencediagram.NotePosition;
 import net.sourceforge.plantuml.sequencediagram.NoteType;
-import net.sourceforge.plantuml.ugraphic.ImageBuilder;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 import net.sourceforge.plantuml.ugraphic.comp.CompressionMode;
-import net.sourceforge.plantuml.ugraphic.comp.TextBlockCompressedOnXorY;
+import net.sourceforge.plantuml.ugraphic.comp.CompressionXorYBuilder;
 
 public class ActivityDiagram3 extends UmlDiagram {
 
@@ -72,7 +72,11 @@ public class ActivityDiagram3 extends UmlDiagram {
 
 	private SwimlaneStrategy swimlaneStrategy;
 
-	private final SwimlanesC swinlanes = new SwimlanesC(getSkinParam(), getPragma());
+	private final Swimlanes swinlanes = new Swimlanes(getSkinParam(), getPragma());
+
+	public ActivityDiagram3(UmlSource source, ISkinSimple skinParam) {
+		super(source, UmlDiagramType.ACTIVITY, skinParam);
+	}
 
 	private void manageSwimlaneStrategy() {
 		if (swimlaneStrategy == null) {
@@ -80,7 +84,7 @@ public class ActivityDiagram3 extends UmlDiagram {
 		}
 	}
 
-	public CommandExecutionResult swimlane(String name, HtmlColor color, Display label) {
+	public CommandExecutionResult swimlane(String name, HColor color, Display label) {
 		if (swimlaneStrategy == null) {
 			swimlaneStrategy = SwimlaneStrategy.SWIMLANE_ALLOWED;
 		}
@@ -104,20 +108,27 @@ public class ActivityDiagram3 extends UmlDiagram {
 		return swinlanes.nextLinkRenderer();
 	}
 
-	public void addActivity(Display activity, BoxStyle style, Url url, Colors colors) {
+	public CommandExecutionResult addActivity(Display activity, BoxStyle style, Url url, Colors colors,
+			Stereotype stereotype) {
 		manageSwimlaneStrategy();
 		final InstructionSimple ins = new InstructionSimple(activity, nextLinkRenderer(),
-				swinlanes.getCurrentSwimlane(), style, url, colors);
-		current().add(ins);
+				swinlanes.getCurrentSwimlane(), style, url, colors, stereotype);
+		final CommandExecutionResult added = current().add(ins);
+		if (added.isOk() == false) {
+			return added;
+		}
 		setNextLinkRendererInternal(LinkRendering.none());
 		manageHasUrl(activity);
 		if (url != null) {
 			hasUrl = true;
 		}
+		return CommandExecutionResult.ok();
+
 	}
 
-	public void addSpot(String spot) {
-		final InstructionSpot ins = new InstructionSpot(spot, nextLinkRenderer(), swinlanes.getCurrentSwimlane());
+	public void addSpot(String spot, HColor color) {
+		final InstructionSpot ins = new InstructionSpot(spot, color, nextLinkRenderer(),
+				swinlanes.getCurrentSwimlane());
 		current().add(ins);
 		setNextLinkRendererInternal(LinkRendering.none());
 		manageSwimlaneStrategy();
@@ -139,7 +150,8 @@ public class ActivityDiagram3 extends UmlDiagram {
 
 	public void start() {
 		manageSwimlaneStrategy();
-		current().add(new InstructionStart(swinlanes.getCurrentSwimlane()));
+		current().add(new InstructionStart(swinlanes.getCurrentSwimlane(), nextLinkRenderer()));
+		setNextLinkRendererInternal(LinkRendering.none());
 	}
 
 	public void stop() {
@@ -169,7 +181,11 @@ public class ActivityDiagram3 extends UmlDiagram {
 		if (last instanceof InstructionWhile == false) {
 			return false;
 		}
-		((InstructionWhile) last).setSpecial(special);
+		final InstructionWhile instructionWhile = (InstructionWhile) last;
+		if (instructionWhile.containsBreak()) {
+			return false;
+		}
+		instructionWhile.setSpecial(special);
 		return true;
 	}
 
@@ -182,51 +198,26 @@ public class ActivityDiagram3 extends UmlDiagram {
 		return new DiagramDescription("activity3");
 	}
 
-	@Override
-	public UmlDiagramType getUmlDiagramType() {
-		return UmlDiagramType.ACTIVITY;
-	}
-
 	@Override
 	protected ImageData exportDiagramInternal(OutputStream os, int index, FileFormatOption fileFormatOption)
 			throws IOException {
 		// BUG42
 		// COMPRESSION
+		final StringBounder stringBounder = fileFormatOption.getDefaultStringBounder(getSkinParam());
+		swinlanes.computeSize(stringBounder);
 		TextBlock result = swinlanes;
-		// result = new TextBlockCompressedOnY(CompressionMode.ON_Y, result);
-		// result = new TextBlockCompressedOnXorY(CompressionMode.ON_X, result);
-		result = new TextBlockCompressedOnXorY(CompressionMode.ON_Y, result);
+
+		result = CompressionXorYBuilder.build(CompressionMode.ON_X, result, stringBounder);
+		result = CompressionXorYBuilder.build(CompressionMode.ON_Y, result, stringBounder);
+
 		result = new TextBlockRecentred(result);
-		final ISkinParam skinParam = getSkinParam();
-		result = new AnnotatedWorker(this, skinParam).addAdd(result);
-		// final Dimension2D dim = TextBlockUtils.getMinMax(result, fileFormatOption.getDefaultStringBounder())
-		// .getDimension();
-		final Dimension2D dim = result.getMinMax(fileFormatOption.getDefaultStringBounder()).getDimension();
-		final double margin = 10;
-		final double dpiFactor = getDpiFactor(fileFormatOption, Dimension2DDouble.delta(dim, 2 * margin, 0));
-
-		final ImageBuilder imageBuilder = new ImageBuilder(getSkinParam(), dpiFactor,
-				fileFormatOption.isWithMetadata() ? getMetadata() : null, getWarningOrError(), margin, margin,
-				getAnimation());
-		imageBuilder.setUDrawable(result);
-
-		return imageBuilder.writeImageTOBEMOVED(fileFormatOption, seed(), os);
-
-	}
-
-	private final double getDpiFactor(FileFormatOption fileFormatOption, final Dimension2D dim) {
-		final double dpiFactor;
-		final Scale scale = getScale();
-		if (scale == null) {
-			dpiFactor = getDpiFactor(fileFormatOption);
-		} else {
-			dpiFactor = scale.getScale(dim.getWidth(), dim.getHeight());
-		}
-		return dpiFactor;
+		return createImageBuilder(fileFormatOption).drawable(result).write(os);
 	}
 
 	public void fork() {
-		final InstructionFork instructionFork = new InstructionFork(current(), nextLinkRenderer(), getSkinParam());
+		manageSwimlaneStrategy();
+		final InstructionFork instructionFork = new InstructionFork(current(), nextLinkRenderer(), getSkinParam(),
+				swinlanes.getCurrentSwimlane());
 		current().add(instructionFork);
 		setNextLinkRendererInternal(LinkRendering.none());
 		setCurrent(instructionFork);
@@ -237,7 +228,7 @@ public class ActivityDiagram3 extends UmlDiagram {
 			final InstructionFork currentFork = (InstructionFork) current();
 			currentFork.manageOutRendering(nextLinkRenderer(), false);
 			setNextLinkRendererInternal(LinkRendering.none());
-			currentFork.forkAgain();
+			currentFork.forkAgain(swinlanes.getCurrentSwimlane());
 			return CommandExecutionResult.ok();
 		}
 		return CommandExecutionResult.error("Cannot find fork");
@@ -246,7 +237,7 @@ public class ActivityDiagram3 extends UmlDiagram {
 	public CommandExecutionResult endFork(ForkStyle forkStyle, String label) {
 		if (current() instanceof InstructionFork) {
 			final InstructionFork currentFork = (InstructionFork) current();
-			currentFork.setStyle(forkStyle, label);
+			currentFork.setStyle(forkStyle, label, swinlanes.getCurrentSwimlane());
 			currentFork.manageOutRendering(nextLinkRenderer(), true);
 			setNextLinkRendererInternal(LinkRendering.none());
 			setCurrent(currentFork.getParent());
@@ -256,7 +247,8 @@ public class ActivityDiagram3 extends UmlDiagram {
 	}
 
 	public void split() {
-		final InstructionSplit instructionSplit = new InstructionSplit(current(), nextLinkRenderer());
+		final InstructionSplit instructionSplit = new InstructionSplit(current(), nextLinkRenderer(),
+				swinlanes.getCurrentSwimlane());
 		setNextLinkRendererInternal(LinkRendering.none());
 		current().add(instructionSplit);
 		setCurrent(instructionSplit);
@@ -273,7 +265,7 @@ public class ActivityDiagram3 extends UmlDiagram {
 
 	public CommandExecutionResult endSplit() {
 		if (current() instanceof InstructionSplit) {
-			((InstructionSplit) current()).endSplit(nextLinkRenderer());
+			((InstructionSplit) current()).endSplit(nextLinkRenderer(), swinlanes.getCurrentSwimlane());
 			setNextLinkRendererInternal(LinkRendering.none());
 			setCurrent(((InstructionSplit) current()).getParent());
 			return CommandExecutionResult.ok();
@@ -281,16 +273,48 @@ public class ActivityDiagram3 extends UmlDiagram {
 		return CommandExecutionResult.error("Cannot find split");
 	}
 
-	public void startIf(Display test, Display whenThen, HtmlColor color) {
+	public void startSwitch(Display test, HColor color) {
+		manageSwimlaneStrategy();
+		final InstructionSwitch instructionSwitch = new InstructionSwitch(swinlanes.getCurrentSwimlane(), current(),
+				test, nextLinkRenderer(), color, getSkinParam());
+		current().add(instructionSwitch);
+		setNextLinkRendererInternal(LinkRendering.none());
+		setCurrent(instructionSwitch);
+	}
+
+	public CommandExecutionResult switchCase(Display labelCase) {
+		if (current() instanceof InstructionSwitch) {
+			final boolean ok = ((InstructionSwitch) current()).switchCase(labelCase, nextLinkRenderer());
+			if (ok == false) {
+				return CommandExecutionResult.error("You cannot put an elseIf here");
+			}
+			setNextLinkRendererInternal(LinkRendering.none());
+			return CommandExecutionResult.ok();
+
+		}
+		return CommandExecutionResult.error("Cannot find switch");
+	}
+
+	public CommandExecutionResult endSwitch() {
+		if (current() instanceof InstructionSwitch) {
+			((InstructionSwitch) current()).endSwitch(nextLinkRenderer());
+			setNextLinkRendererInternal(LinkRendering.none());
+			setCurrent(((InstructionSwitch) current()).getParent());
+			return CommandExecutionResult.ok();
+		}
+		return CommandExecutionResult.error("Cannot find switch");
+	}
+
+	public void startIf(Display test, Display whenThen, HColor color, Url url) {
 		manageSwimlaneStrategy();
 		final InstructionIf instructionIf = new InstructionIf(swinlanes.getCurrentSwimlane(), current(), test,
-				whenThen, nextLinkRenderer(), color, getSkinParam());
+				LinkRendering.none().withDisplay(whenThen), nextLinkRenderer(), color, getSkinParam(), url);
 		current().add(instructionIf);
 		setNextLinkRendererInternal(LinkRendering.none());
 		setCurrent(instructionIf);
 	}
 
-	public CommandExecutionResult elseIf(Display inlabel, Display test, Display whenThen, HtmlColor color) {
+	public CommandExecutionResult elseIf(LinkRendering inlabel, Display test, LinkRendering whenThen, HColor color) {
 		if (current() instanceof InstructionIf) {
 			final boolean ok = ((InstructionIf) current()).elseIf(inlabel, test, whenThen, nextLinkRenderer(), color);
 			if (ok == false) {
@@ -302,7 +326,7 @@ public class ActivityDiagram3 extends UmlDiagram {
 		return CommandExecutionResult.error("Cannot find if");
 	}
 
-	public CommandExecutionResult else2(Display whenElse) {
+	public CommandExecutionResult else2(LinkRendering whenElse) {
 		if (current() instanceof InstructionIf) {
 			final boolean result = ((InstructionIf) current()).swithToElse2(whenElse, nextLinkRenderer());
 			if (result == false) {
@@ -325,10 +349,10 @@ public class ActivityDiagram3 extends UmlDiagram {
 		return CommandExecutionResult.error("Cannot find if");
 	}
 
-	public void startRepeat(HtmlColor color, Display label) {
+	public void startRepeat(HColor color, Display label, BoxStyle boxStyleIn, Colors colors) {
 		manageSwimlaneStrategy();
 		final InstructionRepeat instructionRepeat = new InstructionRepeat(swinlanes.getCurrentSwimlane(), current(),
-				nextLinkRenderer(), color, label);
+				nextLinkRenderer(), color, label, boxStyleIn, colors);
 		current().add(instructionRepeat);
 		setCurrent(instructionRepeat);
 		setNextLinkRendererInternal(LinkRendering.none());
@@ -341,7 +365,7 @@ public class ActivityDiagram3 extends UmlDiagram {
 		if (current() instanceof InstructionRepeat) {
 			final InstructionRepeat instructionRepeat = (InstructionRepeat) current();
 			final LinkRendering back = new LinkRendering(linkColor).withDisplay(linkLabel);
-			instructionRepeat.setTest(label, yes, out, nextLinkRenderer(), back);
+			instructionRepeat.setTest(label, yes, out, nextLinkRenderer(), back, swinlanes.getCurrentSwimlane());
 			setCurrent(instructionRepeat.getParent());
 			this.setNextLinkRendererInternal(LinkRendering.none());
 			return CommandExecutionResult.ok();
@@ -350,21 +374,24 @@ public class ActivityDiagram3 extends UmlDiagram {
 
 	}
 
-	public CommandExecutionResult backwardWhile(Display label) {
+	public CommandExecutionResult backward(Display label, BoxStyle boxStyle, LinkRendering incoming1,
+			LinkRendering incoming2) {
 		manageSwimlaneStrategy();
 		if (current() instanceof InstructionRepeat) {
 			final InstructionRepeat instructionRepeat = (InstructionRepeat) current();
-			// final LinkRendering back = new LinkRendering(linkColor).withDisplay(linkLabel);
-			instructionRepeat.setBackward(label);
-			// setCurrent(instructionRepeat.getParent());
-			// this.setNextLinkRendererInternal(LinkRendering.none());
+			instructionRepeat.setBackward(label, swinlanes.getCurrentSwimlane(), boxStyle, incoming1, incoming2);
+			return CommandExecutionResult.ok();
+		}
+		if (current() instanceof InstructionWhile) {
+			final InstructionWhile instructionWhile = (InstructionWhile) current();
+			instructionWhile.setBackward(label, swinlanes.getCurrentSwimlane(), boxStyle, incoming1, incoming2);
 			return CommandExecutionResult.ok();
 		}
 		return CommandExecutionResult.error("Cannot find repeat");
 
 	}
 
-	public void doWhile(Display test, Display yes, HtmlColor color) {
+	public void doWhile(Display test, Display yes, HColor color) {
 		manageSwimlaneStrategy();
 		final InstructionWhile instructionWhile = new InstructionWhile(swinlanes.getCurrentSwimlane(), current(), test,
 				nextLinkRenderer(), yes, color, getSkinParam());
@@ -374,7 +401,8 @@ public class ActivityDiagram3 extends UmlDiagram {
 
 	public CommandExecutionResult endwhile(Display out) {
 		if (current() instanceof InstructionWhile) {
-			((InstructionWhile) current()).endwhile(nextLinkRenderer(), out);
+			((InstructionWhile) current()).incoming(nextLinkRenderer());
+			((InstructionWhile) current()).outDisplay(out);
 			setNextLinkRendererInternal(LinkRendering.none());
 			setCurrent(((InstructionWhile) current()).getParent());
 			return CommandExecutionResult.ok();
@@ -389,10 +417,11 @@ public class ActivityDiagram3 extends UmlDiagram {
 		return CommandExecutionResult.ok();
 	}
 
-	public void startGroup(Display name, HtmlColor backColor, HtmlColor titleColor, HtmlColor borderColor) {
+	public void startGroup(Display name, HColor backColor, HColor titleColor, HColor borderColor, USymbol type,
+			double roundCorner) {
 		manageSwimlaneStrategy();
 		final InstructionGroup instructionGroup = new InstructionGroup(current(), name, backColor, titleColor,
-				swinlanes.getCurrentSwimlane(), borderColor, nextLinkRenderer());
+				swinlanes.getCurrentSwimlane(), borderColor, nextLinkRenderer(), type, roundCorner);
 		current().add(instructionGroup);
 		setCurrent(instructionGroup);
 	}
@@ -406,30 +435,30 @@ public class ActivityDiagram3 extends UmlDiagram {
 	}
 
 	private void setNextLinkRendererInternal(LinkRendering link) {
-		if (link == null) {
-			throw new IllegalArgumentException();
-		}
 		// System.err.println("setNextLinkRendererInternal=" + link);
-		swinlanes.setNextLinkRenderer(link);
+		swinlanes.setNextLinkRenderer(Objects.requireNonNull(link));
 	}
 
 	private void setNextLink(LinkRendering linkRenderer) {
-		if (linkRenderer == null) {
-			throw new IllegalArgumentException();
-		}
+		Objects.requireNonNull(linkRenderer);
 		// System.err.println("setNextLink=" + linkRenderer);
 		if (current() instanceof InstructionCollection) {
 			final Instruction last = ((InstructionCollection) current()).getLast();
 			if (last instanceof InstructionWhile) {
-				((InstructionWhile) last).afterEndwhile(linkRenderer);
+				((InstructionWhile) last).outColor(linkRenderer.getRainbow());
 			} else if (last instanceof InstructionIf) {
-				((InstructionIf) last).afterEndwhile(linkRenderer);
+				((InstructionIf) last).outColor(linkRenderer);
 			}
 		}
 		this.setNextLinkRendererInternal(linkRenderer);
 	}
 
 	public void setLabelNextArrow(Display label) {
+//		if (current() instanceof InstructionRepeat && ((InstructionRepeat) current()).hasBackward()) {
+//			final InstructionRepeat instructionRepeat = (InstructionRepeat) current();
+//			instructionRepeat.setBackwardArrowLabel(label);
+//			return;
+//		}
 		if (current() instanceof InstructionWhile && ((InstructionWhile) current()).getLast() == null) {
 			((InstructionWhile) current()).overwriteYes(label);
 			return;
