@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -40,6 +40,8 @@ import java.awt.geom.Point2D;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Collections;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
@@ -54,25 +56,27 @@ import net.sourceforge.plantuml.activitydiagram3.ftile.Connection;
 import net.sourceforge.plantuml.activitydiagram3.ftile.ConnectionTranslatable;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Diamond;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Ftile;
+import net.sourceforge.plantuml.activitydiagram3.ftile.FtileEmpty;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileGeometry;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileUtils;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Snake;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Swimlane;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamond;
-import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamondFoo1;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamondInside;
+import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDiamondSquare;
 import net.sourceforge.plantuml.creole.CreoleMode;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.Rainbow;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.graphic.TextBlockUtils;
 import net.sourceforge.plantuml.svek.ConditionStyle;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
+import net.sourceforge.plantuml.ugraphic.UPolygon;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 class FtileRepeat extends AbstractFtile {
 
@@ -105,70 +109,86 @@ class FtileRepeat extends AbstractFtile {
 	}
 
 	public Set<Swimlane> getSwimlanes() {
-		return repeat.getSwimlanes();
+		final Set<Swimlane> result = new HashSet<>(repeat.getSwimlanes());
+		result.addAll(diamond1.getSwimlanes());
+		result.addAll(diamond2.getSwimlanes());
+		return Collections.unmodifiableSet(result);
 	}
 
-	public static Ftile create(LinkRendering backRepeatLinkRendering, Swimlane swimlane, Swimlane swimlaneOut,
-			Ftile backStart, Ftile repeat, Display test, Display yes, Display out, HtmlColor borderColor,
-			HtmlColor backColor, Rainbow arrowColor, Rainbow endRepeatLinkColor, ConditionStyle conditionStyle,
-			ISkinSimple spriteContainer, FontConfiguration fcDiamond, FontConfiguration fcArrow, Ftile backward) {
+	public static Ftile create(Swimlane swimlane, Swimlane swimlaneOut, Ftile entry, Ftile repeat, Display test,
+			Display yes, Display out, HColor borderColor, HColor diamondColor, Rainbow arrowColor,
+			Rainbow endRepeatLinkColor, ConditionStyle conditionStyle, ISkinSimple spriteContainer,
+			FontConfiguration fcDiamond, FontConfiguration fcArrow, Ftile backward, boolean noOut,
+			LinkRendering incoming1, LinkRendering incoming2) {
 
-		final FontConfiguration fontConfiguration1 = conditionStyle == ConditionStyle.INSIDE ? fcDiamond : fcArrow;
+		final FontConfiguration fontConfiguration1 = conditionStyle == ConditionStyle.INSIDE_HEXAGON ? fcDiamond : fcArrow;
 
-		final TextBlock tbTest = (Display.isNull(test) || test.isWhite()) ? TextBlockUtils.empty(0, 0) : test.create(
-				fontConfiguration1, repeat.skinParam().getDefaultTextAlignment(HorizontalAlignment.LEFT),
-				spriteContainer);
+		final TextBlock tbTest = (Display.isNull(test) || test.isWhite()) ? TextBlockUtils.empty(0, 0)
+				: test.create(fontConfiguration1, repeat.skinParam().getDefaultTextAlignment(HorizontalAlignment.LEFT),
+						spriteContainer);
 		final TextBlock yesTb = yes.create(fcArrow, HorizontalAlignment.LEFT, spriteContainer);
 		final TextBlock outTb = out.create(fcArrow, HorizontalAlignment.LEFT, spriteContainer);
 
 		final Ftile diamond1;
-		if (backStart == null) {
-			diamond1 = new FtileDiamond(repeat.skinParam(), backColor, borderColor, swimlane);
+		// assert swimlane == repeat.getSwimlaneIn();
+		if (entry == null) {
+			diamond1 = new FtileDiamond(repeat.skinParam(), diamondColor, borderColor, repeat.getSwimlaneIn());
 		} else {
-			diamond1 = backStart;
+			diamond1 = entry;
 		}
 		final FtileRepeat result;
-		if (conditionStyle == ConditionStyle.INSIDE) {
-			final Ftile diamond2 = new FtileDiamondInside(repeat.skinParam(), backColor, borderColor, swimlaneOut,
-					tbTest).withEast(yesTb).withSouth(outTb);
+		if (conditionStyle == ConditionStyle.INSIDE_HEXAGON) {
+			final Ftile diamond2;
+			if (noOut && Display.isNull(test)) {
+				diamond2 = new FtileEmpty(repeat.skinParam());
+			} else {
+				diamond2 = new FtileDiamondInside(repeat.skinParam(), diamondColor, borderColor, swimlaneOut, tbTest)
+						.withEast(yesTb).withSouth(outTb);
+			}
 			result = new FtileRepeat(repeat, diamond1, diamond2, TextBlockUtils.empty(0, 0), backward);
-		} else if (conditionStyle == ConditionStyle.DIAMOND) {
-			final Ftile diamond2 = new FtileDiamond(repeat.skinParam(), backColor, borderColor, swimlane)
+		} else if (conditionStyle == ConditionStyle.EMPTY_DIAMOND) {
+			final Ftile diamond2 = new FtileDiamond(repeat.skinParam(), diamondColor, borderColor, swimlane)
 					.withEast(tbTest);
 			result = new FtileRepeat(repeat, diamond1, diamond2, tbTest, backward);
-		} else if (conditionStyle == ConditionStyle.FOO1) {
-			final Ftile diamond2 = new FtileDiamondFoo1(repeat.skinParam(), backColor, borderColor, swimlane, tbTest);
+		} else if (conditionStyle == ConditionStyle.INSIDE_DIAMOND) {
+			final Ftile diamond2 = new FtileDiamondSquare(repeat.skinParam(), diamondColor, borderColor, swimlane,
+					tbTest);
 			result = new FtileRepeat(repeat, diamond1, diamond2, TextBlockUtils.empty(0, 0), backward);
 		} else {
 			throw new IllegalStateException();
 		}
 
-		final List<Connection> conns = new ArrayList<Connection>();
+		final List<Connection> conns = new ArrayList<>();
 		final Display in1 = repeat.getInLinkRendering().getDisplay();
-		final TextBlock tbin1 = in1 == null ? null : in1.create(fcArrow, HorizontalAlignment.LEFT, spriteContainer,
-				CreoleMode.SIMPLE_LINE);
+		final TextBlock tbin1 = in1 == null ? null
+				: in1.create7(fcArrow, HorizontalAlignment.LEFT, spriteContainer, CreoleMode.SIMPLE_LINE);
 		conns.add(result.new ConnectionIn(repeat.getInLinkRendering().getRainbow(arrowColor), tbin1));
 
-		final Display backLink1 = backRepeatLinkRendering.getDisplay();
-		final TextBlock tbbackLink1 = backLink1 == null ? null : backLink1.create(fcArrow, HorizontalAlignment.LEFT,
-				spriteContainer, CreoleMode.SIMPLE_LINE);
-		if (repeat.getSwimlaneIn() == repeat.getSwimlaneOut()) {
+		final TextBlock incomingText = incoming1 == null || incoming1.getDisplay() == null ? null
+				: incoming1.getDisplay().create7(fcArrow, HorizontalAlignment.LEFT, spriteContainer,
+						CreoleMode.SIMPLE_LINE);
+
+		if (repeat.getSwimlaneIn() == null || repeat.getSwimlaneIn() == swimlaneOut) {
 			if (backward == null) {
-				conns.add(result.new ConnectionBackSimple(backRepeatLinkRendering.getRainbow(arrowColor), tbbackLink1));
+				conns.add(result.new ConnectionBackSimple(incoming1.getRainbow(arrowColor), incomingText));
 			} else {
-				conns.add(result.new ConnectionBackBackward1(backRepeatLinkRendering.getRainbow(arrowColor),
-						tbbackLink1));
-				conns.add(result.new ConnectionBackBackward2(backRepeatLinkRendering.getRainbow(arrowColor)));
+				final Rainbow rainbow1 = incoming1.getRainbow(arrowColor);
+				conns.add(result.new ConnectionBackBackward1(rainbow1, incomingText));
+				final TextBlock backArrowLabel = incoming2 == null || incoming2.getDisplay() == null ? null
+						: incoming2.getDisplay().create(fcArrow, HorizontalAlignment.LEFT, spriteContainer);
+				final Rainbow rainbow2 = incoming2.getRainbow(arrowColor);
+				conns.add(result.new ConnectionBackBackward2(rainbow2, backArrowLabel));
 			}
 		} else {
-			conns.add(result.new ConnectionBackComplex1(backRepeatLinkRendering.getRainbow(arrowColor)));
-			conns.add(result.new ConnectionBackComplexHorizontalOnly(backRepeatLinkRendering.getRainbow(arrowColor),
-					tbbackLink1));
+			conns.add(result.new ConnectionBackComplex1(incoming1.getRainbow(arrowColor)));
+			// conns.add(result.new
+			// ConnectionBackComplexHorizontalOnly(incoming1.getRainbow(arrowColor),
+			// incomingText));
 		}
 
 		final Display out1 = repeat.getOutLinkRendering().getDisplay();
-		final TextBlock tbout1 = out1 == null ? null : out1.create(fcArrow, HorizontalAlignment.LEFT, spriteContainer,
-				CreoleMode.SIMPLE_LINE);
+		final TextBlock tbout1 = out1 == null ? null
+				: out1.create7(fcArrow, HorizontalAlignment.LEFT, spriteContainer, CreoleMode.SIMPLE_LINE);
 
 		final Rainbow tmpColor = endRepeatLinkColor.withDefault(arrowColor);
 		conns.add(result.new ConnectionOut(tmpColor, tbout1));
@@ -198,8 +218,7 @@ class FtileRepeat extends AbstractFtile {
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
-			snake.setLabel(tbin);
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown()).withLabel(tbin, arrowHorizontalAlignment());
 			snake.addPoint(getP1(stringBounder));
 			snake.addPoint(getP2(stringBounder));
 
@@ -218,13 +237,13 @@ class FtileRepeat extends AbstractFtile {
 		}
 
 		private Point2D getP1(final StringBounder stringBounder) {
-			return getTranslateForRepeat(stringBounder).getTranslated(
-					getFtile1().calculateDimension(stringBounder).getPointOut());
+			return getTranslateForRepeat(stringBounder)
+					.getTranslated(getFtile1().calculateDimension(stringBounder).getPointOut());
 		}
 
 		private Point2D getP2(final StringBounder stringBounder) {
-			return getTranslateDiamond2(stringBounder).getTranslated(
-					getFtile2().calculateDimension(stringBounder).getPointIn());
+			return getTranslateDiamond2(stringBounder)
+					.getTranslated(getFtile2().calculateDimension(stringBounder).getPointIn());
 		}
 
 		public void drawU(UGraphic ug) {
@@ -233,8 +252,8 @@ class FtileRepeat extends AbstractFtile {
 				return;
 			}
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
-			snake.setLabel(tbout);
+			final Snake snake = Snake.create(arrowColor, Arrows.asToDown()).withLabel(tbout,
+					arrowHorizontalAlignment());
 			snake.addPoint(getP1(stringBounder));
 			snake.addPoint(getP2(stringBounder));
 
@@ -246,18 +265,17 @@ class FtileRepeat extends AbstractFtile {
 			if (getFtile1().calculateDimension(stringBounder).hasPointOut() == false) {
 				return;
 			}
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor);
-			snake.setLabel(tbout);
+			final Snake snake = Snake.create(arrowColor);
 			final Point2D mp1a = translate1.getTranslated(getP1(stringBounder));
 			final Point2D mp2b = translate2.getTranslated(getP2(stringBounder));
 			final double middle = (mp1a.getY() + mp2b.getY()) / 2.0;
 			snake.addPoint(mp1a);
 			snake.addPoint(mp1a.getX(), middle);
 			snake.addPoint(mp2b.getX(), middle);
-			// snake.addPoint(mp2b);
 			ug.draw(snake);
 
-			final Snake small = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToDown());
+			final Snake small = Snake.create(arrowColor, Arrows.asToDown()).withLabel(tbout,
+					arrowHorizontalAlignment());
 			small.addPoint(mp2b.getX(), middle);
 			small.addPoint(mp2b);
 			ug.draw(small);
@@ -288,8 +306,6 @@ class FtileRepeat extends AbstractFtile {
 
 		public void drawTranslate(UGraphic ug, UTranslate translate1, UTranslate translate2) {
 			final StringBounder stringBounder = ug.getStringBounder();
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToLeft());
-			snake.emphasizeDirection(Direction.UP);
 			final Dimension2D dimRepeat = repeat.calculateDimension(stringBounder);
 
 			Point2D p1 = getP1(stringBounder);
@@ -299,53 +315,68 @@ class FtileRepeat extends AbstractFtile {
 			final Dimension2D dimDiamond1 = diamond1.calculateDimension(stringBounder);
 			final Dimension2D dimDiamond2 = diamond2.calculateDimension(stringBounder);
 			final double y1 = p1.getY() + dimDiamond2.getHeight() / 2;
-			final double x2 = p2.getX() + dimDiamond1.getWidth();
+			double x2 = p2.getX() + dimDiamond1.getWidth();
 			final double y2 = p2.getY() + dimDiamond1.getHeight() / 2;
 
-			final double xmax = p1.getX() + dimDiamond2.getWidth() / 2 + dimRepeat.getWidth() / 2
+			final double x1_a = p1.getX() + dimDiamond2.getWidth();
+
+			final double x1_b = p1.getX() + dimDiamond2.getWidth() / 2 + dimRepeat.getWidth() / 2
 					+ Diamond.diamondHalfSize;
-			snake.addPoint(xmax, y1);
-			snake.addPoint(xmax, y2);
+
+			UPolygon arrow = Arrows.asToLeft();
+			if (x1_b < x2) {
+				arrow = Arrows.asToRight();
+				x2 = p2.getX();
+			}
+			final Snake snake = Snake.create(arrowColor, arrow).emphasizeDirection(Direction.UP);
+
+			snake.addPoint(x1_a, y1);
+			if (x1_a < x1_b) {
+				snake.addPoint(x1_b, y1);
+				snake.addPoint(x1_b, y2);
+			} else {
+				snake.addPoint(x1_a + 10, y1);
+				snake.addPoint(x1_a + 10, y2);
+
+			}
 			snake.addPoint(x2, y2);
-
 			ug.draw(snake);
 		}
 
 	}
 
-	class ConnectionBackComplexHorizontalOnly extends AbstractConnection {
-		private final Rainbow arrowColor;
-		private final TextBlock tbback;
-
-		public ConnectionBackComplexHorizontalOnly(Rainbow arrowColor, TextBlock tbback) {
-			super(diamond2, diamond2);
-			this.arrowColor = arrowColor;
-			this.tbback = tbback;
-		}
-
-		private Point2D getP1(final StringBounder stringBounder) {
-			return getTranslateDiamond2(stringBounder).getTranslated(new Point2D.Double(0, 0));
-		}
-
-		public void drawU(UGraphic ug) {
-			final StringBounder stringBounder = ug.getStringBounder();
-
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, null);
-			snake.setLabel(tbback);
-			final Dimension2D dimRepeat = repeat.calculateDimension(stringBounder);
-			final Point2D p1 = getP1(stringBounder);
-			final Dimension2D dimDiamond2 = diamond2.calculateDimension(stringBounder);
-			final double x1 = p1.getX() + dimDiamond2.getWidth();
-			final double y1 = p1.getY() + dimDiamond2.getHeight() / 2;
-
-			snake.addPoint(x1, y1);
-			final double xmax = p1.getX() + dimDiamond2.getWidth() / 2 + dimRepeat.getWidth() / 2
-					+ Diamond.diamondHalfSize;
-			snake.addPoint(xmax, y1);
-			ug.draw(snake);
-		}
-
-	}
+//	class ConnectionBackComplexHorizontalOnly extends AbstractConnection {
+//		private final Rainbow arrowColor;
+//		private final TextBlock tbback;
+//
+//		public ConnectionBackComplexHorizontalOnly(Rainbow arrowColor, TextBlock tbback) {
+//			super(diamond2, diamond2);
+//			this.arrowColor = arrowColor;
+//			this.tbback = tbback;
+//		}
+//
+//		private Point2D getP1(final StringBounder stringBounder) {
+//			return getTranslateDiamond2(stringBounder).getTranslated(new Point2D.Double(0, 0));
+//		}
+//
+//		public void drawU(UGraphic ug) {
+//			final StringBounder stringBounder = ug.getStringBounder();
+//
+//			final Snake snake = Snake.create(arrowColor, null).withLabel(tbback, arrowHorizontalAlignment());
+//			final Dimension2D dimRepeat = repeat.calculateDimension(stringBounder);
+//			final Point2D p1 = getP1(stringBounder);
+//			final Dimension2D dimDiamond2 = diamond2.calculateDimension(stringBounder);
+//			final double x1 = p1.getX() + dimDiamond2.getWidth();
+//			final double y1 = p1.getY() + dimDiamond2.getHeight() / 2;
+//
+//			snake.addPoint(x1, y1);
+//			final double xmax = p1.getX() + dimDiamond2.getWidth() / 2 + dimRepeat.getWidth() / 2
+//					+ Diamond.diamondHalfSize;
+//			snake.addPoint(xmax, y1);
+//			ug.draw(snake);
+//		}
+//
+//	}
 
 	class ConnectionBackBackward1 extends AbstractConnection {
 		private final Rainbow arrowColor;
@@ -369,8 +400,7 @@ class FtileRepeat extends AbstractFtile {
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToUp());
-			snake.setLabel(tbback);
+			final Snake snake = Snake.create(arrowColor, Arrows.asToUp()).withLabel(tbback, arrowHorizontalAlignment());
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
 			final Dimension2D dimDiamond2 = diamond2.calculateDimension(stringBounder);
@@ -390,9 +420,11 @@ class FtileRepeat extends AbstractFtile {
 
 	class ConnectionBackBackward2 extends AbstractConnection {
 		private final Rainbow arrowColor;
+		private final TextBlock label;
 
-		public ConnectionBackBackward2(Rainbow arrowColor) {
+		public ConnectionBackBackward2(Rainbow arrowColor, TextBlock label) {
 			super(backward, diamond1);
+			this.label = label;
 			this.arrowColor = arrowColor;
 		}
 
@@ -408,8 +440,11 @@ class FtileRepeat extends AbstractFtile {
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToLeft());
-			snake.emphasizeDirection(Direction.UP);
+			Snake snake = Snake.create(arrowColor, Arrows.asToLeft());
+			// .emphasizeDirection(Direction.UP);
+			if (label != null) {
+				snake = snake.withLabel(label, arrowHorizontalAlignment());
+			}
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
 			final Dimension2D dimDiamond1 = diamond1.calculateDimension(stringBounder);
@@ -427,7 +462,7 @@ class FtileRepeat extends AbstractFtile {
 
 	}
 
-	class ConnectionBackSimple extends AbstractConnection {
+	class ConnectionBackSimple extends AbstractConnection implements ConnectionTranslatable {
 		private final Rainbow arrowColor;
 		private final TextBlock tbback;
 
@@ -448,9 +483,8 @@ class FtileRepeat extends AbstractFtile {
 		public void drawU(UGraphic ug) {
 			final StringBounder stringBounder = ug.getStringBounder();
 
-			final Snake snake = new Snake(arrowHorizontalAlignment(), arrowColor, Arrows.asToLeft());
-			snake.setLabel(tbback);
-			snake.emphasizeDirection(Direction.UP);
+			final Snake snake = Snake.create(arrowColor, Arrows.asToLeft()).emphasizeDirection(Direction.UP)
+					.withLabel(tbback, arrowHorizontalAlignment());
 			final Dimension2D dimTotal = calculateDimensionInternal(stringBounder);
 			final Point2D p1 = getP1(stringBounder);
 			final Point2D p2 = getP2(stringBounder);
@@ -470,6 +504,33 @@ class FtileRepeat extends AbstractFtile {
 			ug.draw(snake);
 		}
 
+		public void drawTranslate(UGraphic ug, UTranslate translate1, UTranslate translate2) {
+			final StringBounder stringBounder = ug.getStringBounder();
+			final Snake snake = Snake.create(arrowColor, Arrows.asToLeft()).emphasizeDirection(Direction.UP)
+					.withLabel(tbback, arrowHorizontalAlignment());
+			final Dimension2D dimRepeat = repeat.calculateDimension(stringBounder);
+
+			Point2D p1 = getP1(stringBounder);
+			Point2D p2 = getP2(stringBounder);
+			p1 = translate1.getTranslated(p1);
+			p2 = translate2.getTranslated(p2);
+			final Dimension2D dimDiamond1 = diamond1.calculateDimension(stringBounder);
+			final Dimension2D dimDiamond2 = diamond2.calculateDimension(stringBounder);
+			final double x1 = p1.getX() + dimDiamond2.getWidth();
+			final double y1 = p1.getY() + dimDiamond2.getHeight() / 2;
+			final double x2 = p2.getX() + dimDiamond1.getWidth();
+			final double y2 = p2.getY() + dimDiamond1.getHeight() / 2;
+
+			snake.addPoint(x1, y1);
+			final double xmax = p1.getX() + dimDiamond2.getWidth() / 2 + dimRepeat.getWidth() / 2
+					+ Diamond.diamondHalfSize;
+			snake.addPoint(xmax, y1);
+			snake.addPoint(xmax, y2);
+			snake.addPoint(x2, y2);
+
+			ug.draw(snake);
+		}
+
 	}
 
 	public void drawU(UGraphic ug) {
@@ -483,7 +544,8 @@ class FtileRepeat extends AbstractFtile {
 
 	}
 
-	public FtileGeometry calculateDimension(StringBounder stringBounder) {
+	@Override
+	protected FtileGeometry calculateDimensionFtile(StringBounder stringBounder) {
 		final Dimension2D dimTotal = calculateDimensionInternal(stringBounder);
 		return new FtileGeometry(dimTotal, getLeft(stringBounder), 0, dimTotal.getHeight());
 	}
@@ -500,8 +562,8 @@ class FtileRepeat extends AbstractFtile {
 		if (backward != null) {
 			width += backward.calculateDimension(stringBounder).getWidth();
 		}
-		final double height = dimDiamond1.getHeight() + dimRepeat.getHeight() + dimDiamond2.getHeight() + 8
-				* Diamond.diamondHalfSize;
+		final double height = dimDiamond1.getHeight() + dimRepeat.getHeight() + dimDiamond2.getHeight()
+				+ 8 * Diamond.diamondHalfSize;
 		return new Dimension2DDouble(width + 2 * Diamond.diamondHalfSize, height);
 
 	}
@@ -534,7 +596,7 @@ class FtileRepeat extends AbstractFtile {
 	private UTranslate getTranslateDiamond1(StringBounder stringBounder) {
 		final Dimension2D dimDiamond1 = diamond1.calculateDimension(stringBounder);
 		final double left = getLeft(stringBounder);
-		return new UTranslate(left - dimDiamond1.getWidth() / 2, 0);
+		return UTranslate.dx(left - dimDiamond1.getWidth() / 2);
 	}
 
 	private UTranslate getTranslateBackward(StringBounder stringBounder) {
