@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -41,6 +41,6 @@ public interface StringCompressor {
 
 	String compress(String s) throws IOException;
 
-	String decompress(String compressed) throws IOException;
+	String decompress(String compressed) throws NoPlantumlCompressionException;
 
 }
\ No newline at end of file
