@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -35,43 +35,45 @@
  */
 package net.sourceforge.plantuml.command;
 
-import java.io.File;
-import java.io.FileInputStream;
+import java.awt.image.BufferedImage;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipInputStream;
 
-import javax.imageio.ImageIO;
-
 import net.sourceforge.plantuml.FileSystem;
 import net.sourceforge.plantuml.FileUtils;
+import net.sourceforge.plantuml.LineLocation;
 import net.sourceforge.plantuml.Log;
-import net.sourceforge.plantuml.UmlDiagram;
+import net.sourceforge.plantuml.TitledDiagram;
+import net.sourceforge.plantuml.command.regex.IRegex;
 import net.sourceforge.plantuml.command.regex.RegexConcat;
 import net.sourceforge.plantuml.command.regex.RegexLeaf;
 import net.sourceforge.plantuml.command.regex.RegexResult;
-import net.sourceforge.plantuml.ugraphic.sprite.Sprite;
-import net.sourceforge.plantuml.ugraphic.sprite.SpriteImage;
-import net.sourceforge.plantuml.ugraphic.sprite.SpriteSvg;
+import net.sourceforge.plantuml.security.ImageIO;
+import net.sourceforge.plantuml.security.SFile;
+import net.sourceforge.plantuml.sprite.Sprite;
+import net.sourceforge.plantuml.sprite.SpriteImage;
+import net.sourceforge.plantuml.sprite.SpriteSvg;
 
-public class CommandSpriteFile extends SingleLineCommand2<UmlDiagram> {
+public class CommandSpriteFile extends SingleLineCommand2<TitledDiagram> {
 
 	public CommandSpriteFile() {
 		super(getRegexConcat());
 	}
 
-	private static RegexConcat getRegexConcat() {
-		return new RegexConcat(new RegexLeaf("^"), //
-				new RegexLeaf("sprite[%s]+\\$?"), //
-				new RegexLeaf("NAME", "([\\p{L}0-9_]+)[%s]*"), //
-				new RegexLeaf("[%s]+"), //
-				new RegexLeaf("FILE", "(.*)"), //
-				new RegexLeaf("$"));
+	private static IRegex getRegexConcat() {
+		return RegexConcat.build(CommandSpriteFile.class.getName(), RegexLeaf.start(), //
+				new RegexLeaf("sprite"), //
+				RegexLeaf.spaceOneOrMore(), //
+				new RegexLeaf("\\$?"), //
+				new RegexLeaf("NAME", "([-%pLN_]+)"), //
+				RegexLeaf.spaceOneOrMore(), //
+				new RegexLeaf("FILE", "(.*)"), RegexLeaf.end());
 	}
 
 	@Override
-	protected CommandExecutionResult executeArg(UmlDiagram system, RegexResult arg) {
+	protected CommandExecutionResult executeArg(TitledDiagram system, LineLocation location, RegexResult arg) {
 		final String src = arg.get("FILE", 0);
 		final Sprite sprite;
 		try {
@@ -84,24 +86,32 @@ public class CommandSpriteFile extends SingleLineCommand2<UmlDiagram> {
 				sprite = new SpriteImage(ImageIO.read(is));
 			} else if (src.contains("~")) {
 				final int idx = src.lastIndexOf("~");
-				final File f = FileSystem.getInstance().getFile(src.substring(0, idx));
+				final SFile f = FileSystem.getInstance().getFile(src.substring(0, idx));
 				if (f.exists() == false) {
-					return CommandExecutionResult.error("File does not exist: " + src);
+					return CommandExecutionResult.error("Cannot read: " + src);
 				}
 				final String name = src.substring(idx + 1);
 				sprite = getImageFromZip(f, name);
 				if (sprite == null) {
-					return CommandExecutionResult.error("No image " + name + " in " + f.getAbsolutePath());
+					return CommandExecutionResult.error("Cannot read: " + src);
 				}
 			} else {
-				final File f = FileSystem.getInstance().getFile(src);
+				final SFile f = FileSystem.getInstance().getFile(src);
 				if (f.exists() == false) {
-					return CommandExecutionResult.error("File does not exist: " + src);
+					return CommandExecutionResult.error("Cannot read: " + src);
 				}
 				if (isSvg(f.getName())) {
-					sprite = new SpriteSvg(f);
+					final String tmp = FileUtils.readSvg(f);
+					if (tmp == null) {
+						return CommandExecutionResult.error("Cannot read: " + src);
+					}
+					sprite = new SpriteSvg(tmp);
 				} else {
-					sprite = new SpriteImage(FileUtils.ImageIO_read(f));
+					final BufferedImage tmp = f.readRasterImageFromFile();
+					if (tmp == null) {
+						return CommandExecutionResult.error("Cannot read: " + src);
+					}
+					sprite = new SpriteImage(tmp);
 				}
 			}
 		} catch (IOException e) {
@@ -112,10 +122,14 @@ public class CommandSpriteFile extends SingleLineCommand2<UmlDiagram> {
 		return CommandExecutionResult.ok();
 	}
 
-	private Sprite getImageFromZip(File f, String name) throws IOException {
+	private Sprite getImageFromZip(SFile f, String name) throws IOException {
+		final InputStream tmp = f.openFile();
+		if (tmp == null) {
+			return null;
+		}
 		ZipInputStream zis = null;
 		try {
-			zis = new ZipInputStream(new FileInputStream(f));
+			zis = new ZipInputStream(tmp);
 			ZipEntry ze = zis.getNextEntry();
 
 			while (ze != null) {
