@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,15 +36,13 @@
 package net.sourceforge.plantuml.core;
 
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.List;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import net.sourceforge.plantuml.BackSlash;
-import net.sourceforge.plantuml.CharSequence2;
-import net.sourceforge.plantuml.CharSequence2Impl;
 import net.sourceforge.plantuml.LineLocation;
+import net.sourceforge.plantuml.StringLocated;
 import net.sourceforge.plantuml.StringUtils;
 import net.sourceforge.plantuml.command.regex.Matcher2;
 import net.sourceforge.plantuml.command.regex.MyPattern;
@@ -55,8 +53,8 @@ import net.sourceforge.plantuml.version.IteratorCounter2;
 import net.sourceforge.plantuml.version.IteratorCounter2Impl;
 
 /**
- * Represents the textual source of some diagram. The source should start with a <code>@startfoo</code> and end with
- * <code>@endfoo</code>.
+ * Represents the textual source of some diagram. The source should start with a
+ * <code>@startfoo</code> and end with <code>@endfoo</code>.
  * <p>
  * So the diagram does not have to be a UML one.
  * 
@@ -65,57 +63,79 @@ import net.sourceforge.plantuml.version.IteratorCounter2Impl;
  */
 final public class UmlSource {
 
-	final private List<String> source;
-	final private List<CharSequence2> source2;
+	final private List<StringLocated> source;
+	final private List<StringLocated> rawSource;
 
-	// final private int startLine;
-	// final private LineLocation startLocation;
+	public UmlSource removeInitialSkinparam() {
+		if (hasInitialSkinparam(source) == false) {
+			return this;
+		}
+		final List<StringLocated> copy = new ArrayList<>(source);
+		while (hasInitialSkinparam(copy)) {
+			copy.remove(1);
+		}
+		return new UmlSource(copy, rawSource);
+	}
+
+	public boolean containsIgnoreCase(String searched) {
+		for (StringLocated s : source) {
+			if (StringUtils.goLowerCase(s.getString()).contains(searched)) {
+				return true;
+			}
+		}
+		return false;
+	}
+
+	private static boolean hasInitialSkinparam(final List<StringLocated> copy) {
+		return copy.size() > 1 && (copy.get(1).getString().startsWith("skinparam ")
+				|| copy.get(1).getString().startsWith("skinparamlocked "));
+	}
+
+	private UmlSource(List<StringLocated> source, List<StringLocated> rawSource) {
+		this.source = source;
+		this.rawSource = rawSource;
+	}
+
+	public UmlSource(List<StringLocated> data, boolean checkEndingBackslash) {
+		this(data, checkEndingBackslash, new ArrayList<StringLocated>());
+	}
 
 	/**
 	 * Build the source from a text.
 	 * 
-	 * @param source
-	 *            the source of the diagram
-	 * @param checkEndingBackslash
-	 *            <code>true</code> if an ending backslash means that a line has to be collapsed with the following one.
-	 * @param startLine
+	 * @param data                 the source of the diagram
+	 * @param checkEndingBackslash <code>true</code> if an ending backslash means
+	 *                             that a line has to be collapsed with the
+	 *                             following one.
 	 */
-	public UmlSource(List<CharSequence2> source, boolean checkEndingBackslash, int startLine) {
-		// this.startLocation = source.get(0).getLocation();
-		// this.startLine = startLine;
-		final List<String> tmp = new ArrayList<String>();
-		final List<CharSequence2> tmp2 = new ArrayList<CharSequence2>();
+	public UmlSource(List<StringLocated> data, boolean checkEndingBackslash, List<StringLocated> rawSource) {
+		this(new ArrayList<StringLocated>(), rawSource);
 
 		if (checkEndingBackslash) {
 			final StringBuilder pending = new StringBuilder();
-			for (CharSequence2 cs : source) {
-				final String s = cs.toString2();
+			for (StringLocated cs : data) {
+				final String s = cs.getString();
 				if (StringUtils.endsWithBackslash(s)) {
 					pending.append(s.substring(0, s.length() - 1));
 				} else {
 					pending.append(s);
-					tmp.add(pending.toString());
-					tmp2.add(new CharSequence2Impl(pending.toString(), cs.getLocation()));
+					this.source.add(new StringLocated(pending.toString(), cs.getLocation()));
 					pending.setLength(0);
 				}
 			}
 		} else {
-			for (CharSequence2 s : source) {
-				tmp.add(s.toString2());
-				tmp2.add(s);
-			}
+			this.source.addAll(data);
 		}
-		this.source = Collections.unmodifiableList(tmp);
-		this.source2 = Collections.unmodifiableList(tmp2);
 	}
 
 	/**
-	 * Retrieve the type of the diagram. This is based on the first line <code>@startfoo</code>.
+	 * Retrieve the type of the diagram. This is based on the first line
+	 * <code>@startfoo</code>.
 	 * 
 	 * @return the type of the diagram.
 	 */
 	public DiagramType getDiagramType() {
-		return DiagramType.getTypeFromArobaseStart(source.get(0));
+		return DiagramType.getTypeFromArobaseStart(source.get(0).getString());
 	}
 
 	/**
@@ -124,7 +144,7 @@ final public class UmlSource {
 	 * @return a iterator that allow counting line number.
 	 */
 	public IteratorCounter2 iterator2() {
-		return new IteratorCounter2Impl(source2);
+		return new IteratorCounter2Impl(source);
 	}
 
 	/**
@@ -134,8 +154,18 @@ final public class UmlSource {
 	 */
 	public String getPlainString() {
 		final StringBuilder sb = new StringBuilder();
-		for (String s : source) {
-			sb.append(s);
+		for (StringLocated s : source) {
+			sb.append(s.getString());
+			sb.append('\r');
+			sb.append(BackSlash.CHAR_NEWLINE);
+		}
+		return sb.toString();
+	}
+
+	public String getRawString() {
+		final StringBuilder sb = new StringBuilder();
+		for (StringLocated s : rawSource) {
+			sb.append(s.getString());
 			sb.append('\r');
 			sb.append(BackSlash.CHAR_NEWLINE);
 		}
@@ -143,34 +173,13 @@ final public class UmlSource {
 	}
 
 	public long seed() {
-		long h = 1125899906842597L; // prime
-		final String string = getPlainString();
-		final int len = string.length();
-
-		for (int i = 0; i < len; i++) {
-			h = 31 * h + string.charAt(i);
-		}
-		return h;
-	}
-
-	/**
-	 * Return a specific line of the diagram description.
-	 * 
-	 * @param n
-	 *            line number, starting at 0
-	 * @return
-	 */
-	private String getLine(int n) {
-		if (n < 0 || n >= source.size()) {
-			return "";
-		}
-		return source.get(n);
+		return StringUtils.seed(getPlainString());
 	}
 
 	public String getLine(LineLocation n) {
-		for (CharSequence2 s : source2) {
+		for (StringLocated s : source) {
 			if (s.getLocation().compareTo(n) == 0) {
-				return s.toString();
+				return s.getString();
 			}
 		}
 		return null;
@@ -185,23 +194,28 @@ final public class UmlSource {
 		return source.size();
 	}
 
+	public boolean getTotalLineCountLessThan5() {
+		return getTotalLineCount() < 5;
+	}
+
 	/**
-	 * Check if a source diagram description is empty. Does not take comment line into account.
+	 * Check if a source diagram description is empty. Does not take comment line
+	 * into account.
 	 * 
 	 * @return <code>true<code> if the diagram does not contain information.
 	 */
 	public boolean isEmpty() {
-		for (String s : source) {
-			if (StartUtils.isArobaseStartDiagram(s)) {
+		for (StringLocated s : source) {
+			if (StartUtils.isArobaseStartDiagram(s.getString())) {
 				continue;
 			}
-			if (StartUtils.isArobaseEndDiagram(s)) {
+			if (StartUtils.isArobaseEndDiagram(s.getString())) {
 				continue;
 			}
-			if (s.matches("\\s*'.*")) {
+			if (s.getString().matches("\\s*'.*")) {
 				continue;
 			}
-			if (StringUtils.trin(s).length() != 0) {
+			if (StringUtils.trin(s.getString()).length() != 0) {
 				return false;
 			}
 		}
@@ -209,14 +223,15 @@ final public class UmlSource {
 	}
 
 	/**
-	 * Retrieve the title, if defined in the diagram source. Never return <code>null</code>.
+	 * Retrieve the title, if defined in the diagram source. Never return
+	 * <code>null</code>.
 	 * 
 	 * @return
 	 */
 	public Display getTitle() {
-		final Pattern2 p = MyPattern.cmpile("(?i)^[%s]*title[%s]+(.+)$");
-		for (String s : source) {
-			final Matcher2 m = p.matcher(s);
+		final Pattern2 p = MyPattern.cmpile("^[%s]*title[%s]+(.+)$");
+		for (StringLocated s : source) {
+			final Matcher2 m = p.matcher(s.getString());
 			final boolean ok = m.matches();
 			if (ok) {
 				return Display.create(m.group(1));
@@ -226,19 +241,16 @@ final public class UmlSource {
 	}
 
 	public boolean isStartDef() {
-		return source.get(0).startsWith("@startdef");
+		return source.get(0).getString().startsWith("@startdef");
 	}
 
 	public String getId() {
 		final Pattern p = Pattern.compile("id=([\\w]+)\\b");
-		final Matcher m = p.matcher(source.get(0));
+		final Matcher m = p.matcher(source.get(0).getString());
 		if (m.find()) {
 			return m.group(1);
 		}
 		return null;
 	}
 
-	// public final int getStartLine() {
-	// return startLine;
-	// }
 }
