@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -30,7 +30,8 @@
  *
  *
  * Original Author:  Arnaud Roques
- * 
+ * Contribution:  Hisashi Miyashita
+ * Contribution:  Miguel Esteves
  *
  */
 package net.sourceforge.plantuml.cucadiagram.entity;
@@ -43,30 +44,32 @@ import java.util.LinkedHashMap;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 
 import net.sourceforge.plantuml.FontParam;
+import net.sourceforge.plantuml.Guillemet;
 import net.sourceforge.plantuml.ISkinParam;
+import net.sourceforge.plantuml.LineLocation;
 import net.sourceforge.plantuml.StringUtils;
 import net.sourceforge.plantuml.Url;
 import net.sourceforge.plantuml.cucadiagram.Bodier;
 import net.sourceforge.plantuml.cucadiagram.Code;
 import net.sourceforge.plantuml.cucadiagram.Display;
+import net.sourceforge.plantuml.cucadiagram.DisplayPositionned;
 import net.sourceforge.plantuml.cucadiagram.EntityPosition;
 import net.sourceforge.plantuml.cucadiagram.EntityUtils;
 import net.sourceforge.plantuml.cucadiagram.GroupRoot;
 import net.sourceforge.plantuml.cucadiagram.GroupType;
 import net.sourceforge.plantuml.cucadiagram.IGroup;
 import net.sourceforge.plantuml.cucadiagram.ILeaf;
+import net.sourceforge.plantuml.cucadiagram.Ident;
 import net.sourceforge.plantuml.cucadiagram.LeafType;
 import net.sourceforge.plantuml.cucadiagram.Link;
-import net.sourceforge.plantuml.cucadiagram.LongCode;
 import net.sourceforge.plantuml.cucadiagram.Stereotag;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
 import net.sourceforge.plantuml.cucadiagram.dot.Neighborhood;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
-import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.USymbol;
 import net.sourceforge.plantuml.graphic.color.ColorType;
 import net.sourceforge.plantuml.graphic.color.Colors;
@@ -75,21 +78,23 @@ import net.sourceforge.plantuml.svek.IEntityImage;
 import net.sourceforge.plantuml.svek.PackageStyle;
 import net.sourceforge.plantuml.svek.SingleStrategy;
 import net.sourceforge.plantuml.ugraphic.UFont;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 import net.sourceforge.plantuml.utils.UniqueSequence;
 
-final class EntityImpl implements ILeaf, IGroup {
+final public class EntityImpl implements ILeaf, IGroup {
 
 	private final EntityFactory entityFactory;
 
 	// Entity
-	private final Code code;
-	private final LongCode longCode;
+	private/* final */Code code;
+	private/* final */Ident ident;
 
 	private Url url;
 
 	private final Bodier bodier;
 	private final String uid = StringUtils.getUid("cl", UniqueSequence.getValue());
 	private Display display = Display.empty();
+	private DisplayPositionned legend = null;
 
 	private LeafType leafType;
 	private Stereotype stereotype;
@@ -99,21 +104,22 @@ final class EntityImpl implements ILeaf, IGroup {
 	private boolean top;
 
 	// Group
-	private Code namespace2;
+	private Code namespace;
 
 	private GroupType groupType;
 
 	// Other
 	private boolean nearDecoration = false;
-	private final Collection<String> portShortNames = new HashSet<String>();
+	private final Collection<String> portShortNames = new HashSet<>();
 	private int xposition;
 	private IEntityImage svekImage;
 
 	private USymbol symbol;
 	private final int rawLayout;
 	private char concurrentSeparator;
+	private LineLocation codeLine;
 
-	private Set<Stereotag> tags = new LinkedHashSet<Stereotag>();
+	private Set<Stereotag> tags = new LinkedHashSet<>();
 
 	public void addStereotag(Stereotag tag) {
 		this.tags.add(tag);
@@ -134,38 +140,41 @@ final class EntityImpl implements ILeaf, IGroup {
 		this.top = top;
 	}
 
-	private EntityImpl(EntityFactory entityFactory, Code code, Bodier bodier, IGroup parentContainer,
-			LongCode longCode, String namespaceSeparator, int rawLayout) {
-		if (code == null) {
-			throw new IllegalArgumentException();
+	private EntityImpl(Ident ident, EntityFactory entityFactory, Code code, Bodier bodier, IGroup parentContainer,
+			String namespaceSeparator, int rawLayout) {
+		this.ident = Objects.requireNonNull(ident);
+		if (entityFactory.namespaceSeparator.V1972()) {
+			code = ident;
 		}
+		this.code = Objects.requireNonNull(code);
 		this.entityFactory = entityFactory;
 		this.bodier = bodier;
-		this.code = code;
 		this.parentContainer = parentContainer;
-		this.longCode = longCode;
 		this.rawLayout = rawLayout;
 	}
 
-	EntityImpl(EntityFactory entityFactory, Code code, Bodier bodier, IGroup parentContainer, LeafType leafType,
-			LongCode longCode, String namespaceSeparator, int rawLayout) {
-		this(entityFactory, code, bodier, parentContainer, longCode, namespaceSeparator, rawLayout);
+	EntityImpl(Ident ident, Code code, EntityFactory entityFactory, Bodier bodier, IGroup parentContainer,
+			LeafType leafType, String namespaceSeparator, int rawLayout) {
+		this(Objects.requireNonNull(ident), entityFactory, code, bodier, parentContainer, namespaceSeparator,
+				rawLayout);
+		// System.err.println("ID for leaf=" + code + " " + ident);
+		// ident.checkSameAs(code, namespaceSeparator);
 		this.leafType = leafType;
 	}
 
-	EntityImpl(EntityFactory entityFactory, Code code, Bodier bodier, IGroup parentContainer, GroupType groupType,
-			Code namespace2, LongCode longCode, String namespaceSeparator, int rawLayout) {
-		this(entityFactory, code, bodier, parentContainer, longCode, namespaceSeparator, rawLayout);
+	EntityImpl(Ident ident, Code code, EntityFactory entityFactory, Bodier bodier, IGroup parentContainer,
+			GroupType groupType, Code namespace, String namespaceSeparator, int rawLayout) {
+		this(Objects.requireNonNull(ident), entityFactory, code, bodier, parentContainer, namespaceSeparator,
+				rawLayout);
+		// System.err.println("ID for group=" + code + " " + ident);
+		ident.checkSameAs(code, namespaceSeparator, entityFactory.namespaceSeparator);
 		this.groupType = groupType;
-		this.namespace2 = namespace2;
+		this.namespace = namespace;
 	}
 
 	public void setContainer(IGroup container) {
 		checkNotGroup();
-		if (container == null) {
-			throw new IllegalArgumentException();
-		}
-		this.parentContainer = container;
+		this.parentContainer = Objects.requireNonNull(container);
 	}
 
 	public LeafType getLeafType() {
@@ -174,9 +183,7 @@ final class EntityImpl implements ILeaf, IGroup {
 
 	public boolean muteToType(LeafType newType, USymbol newSymbol) {
 		checkNotGroup();
-		if (newType == null) {
-			throw new IllegalArgumentException();
-		}
+		Objects.requireNonNull(newType);
 		if (leafType != LeafType.STILL_UNKNOWN) {
 			if (newType == this.leafType) {
 				return true;
@@ -204,7 +211,18 @@ final class EntityImpl implements ILeaf, IGroup {
 		return code;
 	}
 
+	public String getCodeGetName() {
+		return getCode().getName();
+	}
+
+	public Ident getIdent() {
+		return ident;
+	}
+
 	public Display getDisplay() {
+		if (intricated) {
+			return entityFactory.getIntricatedDisplay(ident);
+		}
 		return display;
 	}
 
@@ -225,15 +243,19 @@ final class EntityImpl implements ILeaf, IGroup {
 	}
 
 	public final IGroup getParentContainer() {
-		if (parentContainer == null) {
-			throw new IllegalArgumentException();
-		}
-		return parentContainer;
+		return entityFactory.getParentContainer(ident, parentContainer);
+		// Objects.requireNonNull(parentContainer);
+		// return parentContainer;
 	}
 
 	@Override
 	public String toString() {
-		return code + " " + display + "(" + leafType + ") " + xposition + " " + getUid();
+		// return super.toString() + code + " " + display + "(" + leafType + ")[" +
+		// groupType + "] " + xposition + " "
+		// + getUid();
+		if (entityFactory.namespaceSeparator.V1972())
+			return getUid() + " " + ident + " " + display + "(" + leafType + ")[" + groupType + "]";
+		return "EntityImpl " + code + ident + " " + display + "(" + leafType + ")[" + groupType + "] " + getUid();
 	}
 
 	public final Url getUrl99() {
@@ -300,6 +322,15 @@ final class EntityImpl implements ILeaf, IGroup {
 
 	public EntityPosition getEntityPosition() {
 		checkNotGroup();
+		if (leafType == LeafType.PORT) {
+			return EntityPosition.PORT;
+		}
+		if (leafType == LeafType.PORTIN) {
+			return EntityPosition.PORTIN;
+		}
+		if (leafType == LeafType.PORTOUT) {
+			return EntityPosition.PORTOUT;
+		}
 		if (leafType != LeafType.STATE) {
 			return EntityPosition.NORMAL;
 		}
@@ -310,7 +341,7 @@ final class EntityImpl implements ILeaf, IGroup {
 		if (stereotype == null) {
 			return EntityPosition.NORMAL;
 		}
-		return EntityPosition.fromStereotype(stereotype.getLabel(false));
+		return EntityPosition.fromStereotype(stereotype.getLabel(Guillemet.DOUBLE_COMPARATOR));
 
 	}
 
@@ -329,10 +360,7 @@ final class EntityImpl implements ILeaf, IGroup {
 	}
 
 	public boolean containsLeafRecurse(ILeaf leaf) {
-		if (leaf == null) {
-			throw new IllegalArgumentException();
-		}
-		if (leaf.isGroup()) {
+		if (Objects.requireNonNull(leaf).isGroup()) {
 			throw new IllegalArgumentException();
 		}
 		checkGroup();
@@ -349,8 +377,8 @@ final class EntityImpl implements ILeaf, IGroup {
 
 	public Collection<ILeaf> getLeafsDirect() {
 		checkGroup();
-		final List<ILeaf> result = new ArrayList<ILeaf>();
-		for (ILeaf ent : entityFactory.getLeafsvalues()) {
+		final List<ILeaf> result = new ArrayList<>();
+		for (ILeaf ent : entityFactory.leafs()) {
 			if (ent.isGroup()) {
 				throw new IllegalStateException();
 			}
@@ -363,8 +391,8 @@ final class EntityImpl implements ILeaf, IGroup {
 
 	public Collection<IGroup> getChildren() {
 		checkGroup();
-		final Collection<IGroup> result = new ArrayList<IGroup>();
-		for (IGroup g : entityFactory.getGroupsvalues()) {
+		final Collection<IGroup> result = new ArrayList<>();
+		for (IGroup g : entityFactory.groups()) {
 			if (g != this && g.getParentContainer() == this) {
 				result.add(g);
 			}
@@ -373,6 +401,10 @@ final class EntityImpl implements ILeaf, IGroup {
 	}
 
 	public void moveEntitiesTo(IGroup dest) {
+		if (entityFactory.namespaceSeparator.V1972()) {
+			moveEntitiesTo1972(dest);
+			return;
+		}
 		checkGroup();
 		if (dest.isGroup() == false) {
 			throw new UnsupportedOperationException();
@@ -394,6 +426,70 @@ final class EntityImpl implements ILeaf, IGroup {
 
 	}
 
+	private void moveEntitiesTo1972(IGroup dest) {
+		checkGroup();
+		if (dest.isGroup() == false) {
+			throw new UnsupportedOperationException();
+		}
+		// System.err.println("moveEntitiesTo1972::before1::groups2=" +
+		// entityFactory.groups2());
+		final Ident firstIdent = getIdent();
+		final Ident destIdent = dest.getIdent();
+		// System.err.println("moveEntitiesTo1972::this=" + firstIdent);
+		// System.err.println("moveEntitiesTo1972::dest=" + destIdent);
+		if (destIdent.startsWith(firstIdent) == false) {
+			throw new UnsupportedOperationException();
+		}
+		// System.err.println("moveEntitiesTo1972::before2::groups2=" +
+		// entityFactory.groups2());
+		for (ILeaf ent : new ArrayList<>(entityFactory.leafs2())) {
+			Ident ident = ent.getIdent();
+			if (ident.equals(firstIdent) == false && ident.startsWith(firstIdent)
+					&& ident.startsWith(destIdent) == false) {
+				// System.err.print("moving leaf ident1=" + ident);
+				entityFactory.leafs2.remove(ident);
+				ident = ident.move(firstIdent, destIdent);
+				// System.err.println(" to ident2=" + ident);
+				((EntityImpl) ent).ident = ident;
+				((EntityImpl) ent).code = ident;
+				entityFactory.leafs2.put(ident, ent);
+			}
+		}
+		// System.err.println("moveEntitiesTo1972::before3::groups2=" +
+		// entityFactory.groups2());
+		for (IGroup ent : new ArrayList<>(entityFactory.groups2())) {
+			Ident ident = ent.getIdent();
+			// System.err.println("found=" + ident + " " + ident.startsWith(firstIdent) + "
+			// "
+			// + ident.startsWith(destIdent));
+			if (ident.equals(firstIdent) == false && ident.startsWith(firstIdent)
+					&& ident.startsWith(destIdent) == false) {
+				// System.err.print("moving gr ident1=" + ident);
+				entityFactory.groups2.remove(ident);
+				ident = ident.move(firstIdent, destIdent);
+				// System.err.println(" to ident2=" + ident);
+				((EntityImpl) ent).ident = ident;
+				((EntityImpl) ent).code = ident;
+				entityFactory.groups2.put(ident, ent);
+				// System.err.println("-->groups2=" + entityFactory.groups2());
+			}
+		}
+		// System.err.println("moveEntitiesTo1972::after::groups2=" +
+		// entityFactory.groups2());
+		// for (IGroup g : dest.getChildren()) {
+		// // ((EntityImpl) g).parentContainer = dest;
+		// throw new IllegalStateException();
+		// }
+		//
+		// for (IGroup g : getChildren()) {
+		// if (g == dest) {
+		// continue;
+		// }
+		// ((EntityImpl) g).parentContainer = dest;
+		// }
+
+	}
+
 	public int size() {
 		checkGroup();
 		return getLeafsDirect().size();
@@ -404,9 +500,9 @@ final class EntityImpl implements ILeaf, IGroup {
 		return groupType;
 	}
 
-	public Code getNamespace2() {
+	public Code getNamespace() {
 		checkGroup();
-		return namespace2;
+		return namespace;
 	}
 
 	public PackageStyle getPackageStyle() {
@@ -421,6 +517,7 @@ final class EntityImpl implements ILeaf, IGroup {
 		if (groupType != null && leafType != null) {
 			throw new IllegalStateException();
 		}
+		assert groupType == null || leafType == null;
 		if (groupType != null) {
 			return true;
 		}
@@ -437,16 +534,25 @@ final class EntityImpl implements ILeaf, IGroup {
 		this.svekImage = img;
 		this.url = null;
 
-		for (final Link link : new ArrayList<Link>(entityFactory.getLinks())) {
+		for (final Link link : new ArrayList<>(entityFactory.getLinks())) {
 			if (EntityUtils.isPureInnerLink12(this, link)) {
 				entityFactory.removeLink(link);
 			}
 		}
 
-		entityFactory.removeGroup(this.getCode());
-		for (ILeaf ent : new ArrayList<ILeaf>(entityFactory.getLeafsvalues())) {
-			if (this != ent && this == ent.getParentContainer()) {
-				entityFactory.removeLeaf(ent.getCode());
+		if (entityFactory.namespaceSeparator.V1972()) {
+			entityFactory.removeGroup(getIdent());
+			for (ILeaf ent : new ArrayList<>(entityFactory.leafs())) {
+				if (this != ent && getIdent().equals(ent.getIdent().parent())) {
+					entityFactory.removeLeaf(ent.getIdent());
+				}
+			}
+		} else {
+			entityFactory.removeGroup(getCodeGetName());
+			for (ILeaf ent : new ArrayList<>(entityFactory.leafs())) {
+				if (this != ent && this == ent.getParentContainer()) {
+					entityFactory.removeLeaf(ent.getCodeGetName());
+				}
 			}
 		}
 
@@ -455,12 +561,12 @@ final class EntityImpl implements ILeaf, IGroup {
 		this.leafType = leafType;
 	}
 
-	void muteToGroup(Code namespace2, GroupType groupType, IGroup parentContainer) {
+	void muteToGroup(Code namespaceNew, GroupType groupType, IGroup parentContainer) {
 		checkNotGroup();
 		if (parentContainer.isGroup() == false) {
 			throw new IllegalArgumentException();
 		}
-		this.namespace2 = namespace2;
+		this.namespace = namespaceNew;
 		this.groupType = groupType;
 		this.leafType = null;
 		this.parentContainer = parentContainer;
@@ -470,9 +576,9 @@ final class EntityImpl implements ILeaf, IGroup {
 		if (getLeafType() == LeafType.CIRCLE) {
 			return USymbol.INTERFACE;
 		}
-		if (symbol != null && stereotype != null && stereotype.getSprite() != null) {
-			return symbol.withStereoAlignment(HorizontalAlignment.RIGHT);
-		}
+		// if (symbol != null && stereotype != null && stereotype.getSprite() != null) {
+		// return symbol.withStereoAlignment(HorizontalAlignment.RIGHT);
+		// }
 		return symbol;
 	}
 
@@ -481,7 +587,7 @@ final class EntityImpl implements ILeaf, IGroup {
 	}
 
 	public SingleStrategy getSingleStrategy() {
-		return SingleStrategy.SQUARRE;
+		return SingleStrategy.SQUARE;
 	}
 
 	public boolean isHidden() {
@@ -520,13 +626,13 @@ final class EntityImpl implements ILeaf, IGroup {
 		}
 		return isRemovedInternal();
 	}
-	
+
 	private boolean isRemovedInternal() {
 		if (isGroup()) {
 			if (entityFactory.isRemoved(this)) {
 				return true;
 			}
-			if (getLeafsDirect().size() == 0) {
+			if (getLeafsDirect().size() == 0 && getChildren().size() == 0) {
 				return false;
 			}
 			for (ILeaf leaf : getLeafsDirect()) {
@@ -544,7 +650,17 @@ final class EntityImpl implements ILeaf, IGroup {
 		return entityFactory.isRemoved(this);
 	}
 
-
+	public boolean isAloneAndUnlinked() {
+		if (isGroup()) {
+			return false;
+		}
+		for (Link link : entityFactory.getLinks()) {
+			if (link.contains(this) && link.getType().isInvisible() == false) {
+				return false;
+			}
+		}
+		return true;
+	}
 
 	private int layer;
 
@@ -559,10 +675,6 @@ final class EntityImpl implements ILeaf, IGroup {
 		}
 	}
 
-	public LongCode getLongCode() {
-		return longCode;
-	}
-
 	private FontParam getTitleFontParam() {
 		if (symbol != null) {
 			return symbol.getFontParam();
@@ -572,7 +684,7 @@ final class EntityImpl implements ILeaf, IGroup {
 
 	public FontConfiguration getFontConfigurationForTitle(final ISkinParam skinParam) {
 		final FontParam fontParam = getTitleFontParam();
-		final HtmlColor fontHtmlColor = skinParam.getFontHtmlColor(getStereotype(), fontParam, FontParam.PACKAGE);
+		final HColor fontHtmlColor = skinParam.getFontHtmlColor(getStereotype(), fontParam, FontParam.PACKAGE);
 		final UFont font = skinParam.getFont(getStereotype(), true, fontParam, FontParam.PACKAGE);
 		final FontConfiguration fontConfiguration = new FontConfiguration(font, fontHtmlColor,
 				skinParam.getHyperlinkColor(), skinParam.useUnderlineForHyperlink(), skinParam.getTabSize());
@@ -621,7 +733,7 @@ final class EntityImpl implements ILeaf, IGroup {
 		this.colors = colors;
 	}
 
-	public void setSpecificColorTOBEREMOVED(ColorType type, HtmlColor color) {
+	public void setSpecificColorTOBEREMOVED(ColorType type, HColor color) {
 		if (color != null) {
 			this.colors = colors.add(type, color);
 		}
@@ -629,7 +741,8 @@ final class EntityImpl implements ILeaf, IGroup {
 
 	public Collection<String> getPortShortNames() {
 		checkNotGroup();
-		return Collections.unmodifiableCollection(portShortNames);
+		// return Collections.unmodifiableCollection(portShortNames);
+		return portShortNames;
 	}
 
 	public void addPortShortName(String portShortName) {
@@ -646,4 +759,49 @@ final class EntityImpl implements ILeaf, IGroup {
 	public VisibilityModifier getVisibilityModifier() {
 		return visibility;
 	}
+
+	public void setLegend(DisplayPositionned legend) {
+		checkGroup();
+		this.legend = legend;
+	}
+
+	public DisplayPositionned getLegend() {
+		return legend;
+	}
+
+	private boolean intricated;
+
+	public void setIntricated(boolean intricated) {
+		this.intricated = intricated;
+
+	}
+
+	private IGroup originalGroup;
+
+	public void setOriginalGroup(IGroup originalGroup) {
+		this.originalGroup = originalGroup;
+		this.legend = originalGroup.getLegend();
+	}
+
+	public IGroup getOriginalGroup() {
+		return originalGroup;
+	}
+
+	private boolean together;
+
+	public void setThisIsTogether() {
+		this.together = true;
+	}
+
+	public String getCodeLine() {
+		if (this.codeLine == null) {
+			return null;
+		}
+		return "" + this.codeLine.getPosition();
+	}
+
+	public void setCodeLine(LineLocation codeLine) {
+		this.codeLine = codeLine;
+	}
+
 }
