@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -54,11 +54,14 @@ import net.sourceforge.plantuml.command.regex.RegexResult;
 import net.sourceforge.plantuml.cucadiagram.Code;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.cucadiagram.ILeaf;
+import net.sourceforge.plantuml.cucadiagram.Ident;
 import net.sourceforge.plantuml.cucadiagram.LeafType;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
 import net.sourceforge.plantuml.graphic.USymbol;
 import net.sourceforge.plantuml.graphic.color.ColorParser;
 import net.sourceforge.plantuml.graphic.color.ColorType;
+import net.sourceforge.plantuml.graphic.color.Colors;
+import net.sourceforge.plantuml.ugraphic.color.NoSuchColorException;
 
 public class CommandCreateElementMultilines extends CommandMultilines2<AbstractEntityDiagram> {
 
@@ -76,47 +79,55 @@ public class CommandCreateElementMultilines extends CommandMultilines2<AbstractE
 	@Override
 	public String getPatternEnd() {
 		if (type == 0) {
-			return "(?i)^(.*)[%g]$";
+			return "^(.*)[%g]$";
 		}
 		if (type == 1) {
-			return "(?i)^(.*)\\]$";
+			return "^([^\\[\\]]*)\\]$";
 		}
 		throw new IllegalArgumentException();
 	}
 
 	private static RegexConcat getRegexConcat(int type) {
 		if (type == 0) {
-			return new RegexConcat(new RegexLeaf("^"), //
+			return RegexConcat.build(CommandCreateElementMultilines.class.getName() + type, RegexLeaf.start(), //
 					new RegexLeaf("TYPE", "(" + CommandCreateElementFull.ALL_TYPES + ")[%s]+"), //
-					new RegexLeaf("CODE", "([\\p{L}0-9_.]+)"), //
-					new RegexLeaf("[%s]*"), //
+					new RegexLeaf("CODE", "([%pLN_.]+)"), //
+					RegexLeaf.spaceZeroOrMore(), //
 					new RegexLeaf("STEREO", "(\\<\\<.+\\>\\>)?"), //
-					new RegexLeaf("[%s]*"), //
+					RegexLeaf.spaceZeroOrMore(), //
 					new RegexLeaf("URL", "(" + UrlBuilder.getRegexp() + ")?"), //
-					new RegexLeaf("[%s]*"), //
+					RegexLeaf.spaceZeroOrMore(), //
 					ColorParser.exp1(), //
-					new RegexLeaf("[%s]*"), //
-					new RegexLeaf("DESC", "as[%s]*[%g]([^%g]*)$"));
+					RegexLeaf.spaceZeroOrMore(), //
+					new RegexLeaf("as"), //
+					RegexLeaf.spaceZeroOrMore(), //
+					new RegexLeaf("[%g]"), //
+					new RegexLeaf("DESC", "([^%g]*)"), //
+					RegexLeaf.end());
 		}
 		if (type == 1) {
-			return new RegexConcat(new RegexLeaf("^"), //
+			return RegexConcat.build(CommandCreateElementMultilines.class.getName() + type, RegexLeaf.start(), //
 					new RegexLeaf("TYPE", "(" + CommandCreateElementFull.ALL_TYPES + ")[%s]+"), //
-					new RegexLeaf("CODE", "([\\p{L}0-9_.]+)"), //
-					new RegexLeaf("[%s]*"), //
+					new RegexLeaf("CODE", "([%pLN_.]+)"), //
+					RegexLeaf.spaceZeroOrMore(), //
 					new RegexLeaf("STEREO", "(\\<\\<.+\\>\\>)?"), //
-					new RegexLeaf("[%s]*"), //
+					RegexLeaf.spaceZeroOrMore(), //
 					new RegexLeaf("URL", "(" + UrlBuilder.getRegexp() + ")?"), //
-					new RegexLeaf("[%s]*"), //
+					RegexLeaf.spaceZeroOrMore(), //
 					ColorParser.exp1(), //
-					new RegexLeaf("[%s]*"), //
-					new RegexLeaf("DESC", "\\[(.*)$"));
+					RegexLeaf.spaceZeroOrMore(), //
+					new RegexLeaf("\\["), //
+					new RegexLeaf("DESC", "(.*)"), //
+					RegexLeaf.end());
 		}
 		throw new IllegalArgumentException();
 	}
 
-	protected CommandExecutionResult executeNow(AbstractEntityDiagram diagram, BlocLines lines) {
-		lines = lines.trim(false);
-		final RegexResult line0 = getStartingPattern().matcher(StringUtils.trin(lines.getFirst499()));
+	@Override
+	protected CommandExecutionResult executeNow(AbstractEntityDiagram diagram, BlocLines lines)
+			throws NoSuchColorException {
+		lines = lines.trimSmart(1);
+		final RegexResult line0 = getStartingPattern().matcher(lines.getFirst().getTrimmed().getString());
 		final String symbol = StringUtils.goUpperCase(line0.get("TYPE", 0));
 		final LeafType type;
 		USymbol usymbol;
@@ -125,16 +136,17 @@ public class CommandCreateElementMultilines extends CommandMultilines2<AbstractE
 			type = LeafType.USECASE;
 			usymbol = null;
 		} else {
-			usymbol = USymbol.getFromString(symbol);
+			usymbol = USymbol.fromString(symbol, diagram.getSkinParam().actorStyle(),
+					diagram.getSkinParam().componentStyle(), diagram.getSkinParam().packageStyle());
 			if (usymbol == null) {
 				throw new IllegalStateException();
 			}
 			type = LeafType.DESCRIPTION;
 		}
 
-		final Code code = Code.of(line0.get("CODE", 0));
-		final List<String> lineLast = StringUtils.getSplit(MyPattern.cmpile(getPatternEnd()), lines.getLast499()
-				.toString());
+		final String idShort = line0.get("CODE", 0);
+		final List<String> lineLast = StringUtils.getSplit(MyPattern.cmpile(getPatternEnd()),
+				lines.getLast().getTrimmed().getString());
 		lines = lines.subExtract(1, 1);
 		Display display = lines.toDisplay();
 		final String descStart = line0.get("DESC", 0);
@@ -148,20 +160,22 @@ public class CommandCreateElementMultilines extends CommandMultilines2<AbstractE
 
 		final String stereotype = line0.get("STEREO", 0);
 
-		if (CommandCreateElementFull.existsWithBadType(diagram, code, type, usymbol)) {
-			return CommandExecutionResult.error("This element (" + code.getFullName() + ") is already defined");
+		final Ident ident = diagram.buildLeafIdent(idShort);
+		final Code code = diagram.V1972() ? ident : diagram.buildCode(idShort);
+		if (CommandCreateElementFull.existsWithBadType3(diagram, code, ident, type, usymbol)) {
+			return CommandExecutionResult.error("This element (" + code.getName() + ") is already defined");
 		}
-		final ILeaf result = diagram.createLeaf(code, display, type, usymbol);
+		final ILeaf result = diagram.createLeaf(ident, code, display, type, usymbol);
 		if (result == null) {
-			return CommandExecutionResult.error("This element (" + code.getFullName() + ") is already defined");
+			return CommandExecutionResult.error("This element (" + code.getName() + ") is already defined");
 		}
 		result.setUSymbol(usymbol);
 		if (stereotype != null) {
-			result.setStereotype(new Stereotype(stereotype, diagram.getSkinParam().getCircledCharacterRadius(), diagram
-					.getSkinParam().getFont(null, false, FontParam.CIRCLED_CHARACTER), diagram.getSkinParam()
-					.getIHtmlColorSet()));
+			result.setStereotype(new Stereotype(stereotype, diagram.getSkinParam().getCircledCharacterRadius(),
+					diagram.getSkinParam().getFont(null, false, FontParam.CIRCLED_CHARACTER),
+					diagram.getSkinParam().getIHtmlColorSet()));
 		}
-		
+
 		final String urlString = line0.get("URL", 0);
 		if (urlString != null) {
 			final UrlBuilder urlBuilder = new UrlBuilder(diagram.getSkinParam().getValue("topurl"), ModeUrl.STRICT);
@@ -169,10 +183,19 @@ public class CommandCreateElementMultilines extends CommandMultilines2<AbstractE
 			result.addUrl(url);
 		}
 
-
-		result.setSpecificColorTOBEREMOVED(ColorType.BACK,
-				diagram.getSkinParam().getIHtmlColorSet().getColorIfValid(line0.get("COLOR", 0)));
+		// final HColor backColor =
+		// diagram.getSkinParam().getIHtmlColorSet().getColorIfValid(line0.get("COLOR",
+		// 0));
+		final Colors colors = color().getColor(diagram.getSkinParam().getThemeStyle(), line0,
+				diagram.getSkinParam().getIHtmlColorSet());
+		result.setColors(colors);
+		// result.setSpecificColorTOBEREMOVED(ColorType.BACK, backColor);
 
 		return CommandExecutionResult.ok();
 	}
+
+	private static ColorParser color() {
+		return ColorParser.simpleColor(ColorType.BACK);
+	}
+
 }
