@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,21 +36,24 @@
 package net.sourceforge.plantuml.graphic;
 
 import net.sourceforge.plantuml.Url;
-import net.sourceforge.plantuml.ugraphic.ColorMapper;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
+import net.sourceforge.plantuml.ugraphic.UGroupType;
 import net.sourceforge.plantuml.ugraphic.UParam;
 import net.sourceforge.plantuml.ugraphic.UShape;
+import net.sourceforge.plantuml.ugraphic.color.ColorMapper;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 public abstract class UGraphicDelegator implements UGraphic {
 
 	final private UGraphic ug;
-	
-	public final boolean matchesProperty(String propertyName) {
-		return ug.matchesProperty(propertyName);
+
+	@Override
+	public String toString() {
+		return super.toString() + " " + getUg().toString();
 	}
 
-	public double dpiFactor() {
-		return ug.dpiFactor();
+	public final boolean matchesProperty(String propertyName) {
+		return ug.matchesProperty(propertyName);
 	}
 
 	public UGraphicDelegator(UGraphic ug) {
@@ -77,16 +80,29 @@ public abstract class UGraphicDelegator implements UGraphic {
 		ug.startUrl(url);
 	}
 
-	public void closeAction() {
-		ug.closeAction();
+	public void closeUrl() {
+		ug.closeUrl();
+	}
+
+	public void startGroup(UGroupType type, String ident) {
+		ug.startGroup(type, ident);
+	}
+
+	public void closeGroup() {
+		ug.closeGroup();
 	}
 
 	protected UGraphic getUg() {
 		return ug;
 	}
-	
+
 	public void flushUg() {
 		ug.flushUg();
 	}
 
+	@Override
+	public HColor getDefaultBackground() {
+		return ug.getDefaultBackground();
+	}
+
 }
\ No newline at end of file
