@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -60,9 +60,11 @@ import net.sourceforge.plantuml.cucadiagram.dot.GraphvizUtils;
 import net.sourceforge.plantuml.cucadiagram.dot.GraphvizVersion;
 import net.sourceforge.plantuml.cucadiagram.dot.GraphvizVersions;
 import net.sourceforge.plantuml.cucadiagram.dot.ProcessState;
+import net.sourceforge.plantuml.cucadiagram.entity.EntityFactory;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.posimo.Moveable;
+import net.sourceforge.plantuml.security.SFile;
 import net.sourceforge.plantuml.vizjs.GraphvizJs;
 import net.sourceforge.plantuml.vizjs.GraphvizJsRuntimeException;
 
@@ -70,7 +72,7 @@ public class DotStringFactory implements Moveable {
 
 	private final Bibliotekon bibliotekon = new Bibliotekon();
 
-	final private Set<String> rankMin = new HashSet<String>();
+	final private Set<String> rankMin = new HashSet<>();
 
 	private final ColorSequence colorSequence;
 	private final Cluster root;
@@ -79,6 +81,7 @@ public class DotStringFactory implements Moveable {
 	private final UmlDiagramType umlDiagramType;
 	private final ISkinParam skinParam;
 	private final DotMode dotMode;
+	private DotSplines dotSplines;
 
 	private final StringBounder stringBounder;
 
@@ -104,8 +107,8 @@ public class DotStringFactory implements Moveable {
 		this.current = root;
 	}
 
-	public void addShape(Shape shape) {
-		current.addShape(shape);
+	public void addNode(SvekNode node) {
+		current.addNode(node);
 	}
 
 	private void printMinRanking(StringBuilder sb) {
@@ -123,7 +126,7 @@ public class DotStringFactory implements Moveable {
 
 	private double getHorizontalDzeta() {
 		double max = 0;
-		for (Line l : bibliotekon.allLines()) {
+		for (SvekLine l : bibliotekon.allLines()) {
 			final double c = l.getHorizontalDzeta(stringBounder);
 			if (c > max) {
 				max = c;
@@ -134,7 +137,7 @@ public class DotStringFactory implements Moveable {
 
 	private double getVerticalDzeta() {
 		double max = 0;
-		for (Line l : bibliotekon.allLines()) {
+		for (SvekLine l : bibliotekon.allLines()) {
 			final double c = l.getVerticalDzeta(stringBounder);
 			if (c > max) {
 				max = c;
@@ -188,12 +191,13 @@ public class DotStringFactory implements Moveable {
 		// SvekUtils.println(sb);
 		// }
 
-		final DotSplines dotSplines = skinParam.getDotSplines();
+		dotSplines = skinParam.getDotSplines();
 		if (dotSplines == DotSplines.POLYLINE) {
 			sb.append("splines=polyline;");
 			SvekUtils.println(sb);
 		} else if (dotSplines == DotSplines.ORTHO) {
 			sb.append("splines=ortho;");
+			sb.append("forcelabels=true;");
 			SvekUtils.println(sb);
 		}
 
@@ -205,15 +209,15 @@ public class DotStringFactory implements Moveable {
 		manageMinMaxCluster(sb);
 
 		root.printCluster1(sb, bibliotekon.allLines(), stringBounder);
-		for (Line line : bibliotekon.lines0()) {
-			line.appendLine(getGraphvizVersion(), sb, dotMode);
+		for (SvekLine line : bibliotekon.lines0()) {
+			line.appendLine(getGraphvizVersion(), sb, dotMode, dotSplines);
 		}
 		root.fillRankMin(rankMin);
 		root.printCluster2(sb, bibliotekon.allLines(), stringBounder, dotMode, getGraphvizVersion(), umlDiagramType);
 		printMinRanking(sb);
 
-		for (Line line : bibliotekon.lines1()) {
-			line.appendLine(getGraphvizVersion(), sb, dotMode);
+		for (SvekLine line : bibliotekon.lines1()) {
+			line.appendLine(getGraphvizVersion(), sb, dotMode, dotSplines);
 		}
 		SvekUtils.println(sb);
 		sb.append("}");
@@ -222,8 +226,8 @@ public class DotStringFactory implements Moveable {
 	}
 
 	private void manageMinMaxCluster(final StringBuilder sb) {
-		final List<String> minPointCluster = new ArrayList<String>();
-		final List<String> maxPointCluster = new ArrayList<String>();
+		final List<String> minPointCluster = new ArrayList<>();
+		final List<String> maxPointCluster = new ArrayList<>();
 		for (Cluster cluster : bibliotekon.allCluster()) {
 			final String minPoint = cluster.getMinPoint(umlDiagramType);
 			if (minPoint != null) {
@@ -294,7 +298,7 @@ public class DotStringFactory implements Moveable {
 		String dotString = createDotString(dotOptions);
 
 		if (basefile != null) {
-			final File f = basefile.getTraceFile("svek.dot");
+			final SFile f = basefile.getTraceFile("svek.dot");
 			SvekUtils.traceString(f, dotString);
 		}
 
@@ -322,7 +326,7 @@ public class DotStringFactory implements Moveable {
 		final String s = new String(result, "UTF-8");
 
 		if (basefile != null) {
-			final File f = basefile.getTraceFile("svek.svg");
+			final SFile f = basefile.getTraceFile("svek.svg");
 			SvekUtils.traceString(f, s);
 		}
 
@@ -343,7 +347,8 @@ public class DotStringFactory implements Moveable {
 		return graphviz.getDotExe();
 	}
 
-	public ClusterPosition solve(String svg) throws IOException, InterruptedException {
+	public void solve(boolean mergeIntricated, EntityFactory entityFactory, final String svg)
+			throws IOException, InterruptedException {
 		if (svg.length() == 0) {
 			throw new EmptySvgException();
 		}
@@ -353,92 +358,106 @@ public class DotStringFactory implements Moveable {
 		if (mGraph.find() == false) {
 			throw new IllegalStateException();
 		}
-		final int fullWidth = Integer.parseInt(mGraph.group(1));
+//		final int fullWidth = Integer.parseInt(mGraph.group(1));
 		final int fullHeight = Integer.parseInt(mGraph.group(2));
 
-		final MinFinder corner1 = new MinFinder();
+//		final MinFinder corner1 = new MinFinder();
 
-		for (Shape sh : bibliotekon.allShapes()) {
-			int idx = svg.indexOf("<title>" + sh.getUid() + "</title>");
-			if (sh.getType() == ShapeType.RECTANGLE || sh.getType() == ShapeType.RECTANGLE_HTML_FOR_PORTS
-					|| sh.getType() == ShapeType.FOLDER || sh.getType() == ShapeType.DIAMOND) {
-				final List<Point2D.Double> points = SvekUtils.extractPointsList(svg, idx, fullHeight);
-				final double minX = SvekUtils.getMinX(points);
+		final Point2DFunction move = new YDelta(fullHeight);
+		final SvgResult svgResult = new SvgResult(svg, move);
+		for (SvekNode node : bibliotekon.allNodes()) {
+			int idx = svg.indexOf("<title>" + node.getUid() + "</title>");
+			if (node.getType() == ShapeType.RECTANGLE || node.getType() == ShapeType.RECTANGLE_HTML_FOR_PORTS
+					|| node.getType() == ShapeType.RECTANGLE_WITH_CIRCLE_INSIDE || node.getType() == ShapeType.FOLDER
+					|| node.getType() == ShapeType.DIAMOND) {
+				final List<Point2D.Double> points = svgResult.substring(idx).extractList(SvgResult.POINTS_EQUALS);
 				final double minY = SvekUtils.getMinY(points);
-				corner1.manage(minX, minY);
-				sh.moveSvek(minX, minY);
-			} else if (sh.getType() == ShapeType.ROUND_RECTANGLE) {
+				final double overscanX = node.getOverscanX(stringBounder);
+				final double minX = SvekUtils.getMinX(points);
+//				corner1.manage(minX - overscanX, minY);
+				node.moveSvek(minX, minY);
+			} else if (node.getType() == ShapeType.ROUND_RECTANGLE) {
 				final int idx2 = svg.indexOf("d=\"", idx + 1);
 				idx = svg.indexOf("points=\"", idx + 1);
 				final List<Point2D.Double> points;
 				if (idx2 != -1 && (idx == -1 || idx2 < idx)) {
 					// GraphViz 2.30
-					points = SvekUtils.extractD(svg, idx2, fullHeight);
+					points = svgResult.substring(idx2).extractList(SvgResult.D_EQUALS);
 				} else {
-					points = SvekUtils.extractPointsList(svg, idx, fullHeight);
+					points = svgResult.substring(idx).extractList(SvgResult.POINTS_EQUALS);
 					for (int i = 0; i < 3; i++) {
 						idx = svg.indexOf("points=\"", idx + 1);
-						points.addAll(SvekUtils.extractPointsList(svg, idx, fullHeight));
+						points.addAll(svgResult.substring(idx).extractList(SvgResult.POINTS_EQUALS));
 					}
 				}
 				final double minX = SvekUtils.getMinX(points);
 				final double minY = SvekUtils.getMinY(points);
-				corner1.manage(minX, minY);
-				sh.moveSvek(minX, minY);
-			} else if (sh.getType() == ShapeType.OCTAGON) {
+//				corner1.manage(minX, minY);
+				node.moveSvek(minX, minY);
+			} else if (node.getType() == ShapeType.OCTAGON || node.getType() == ShapeType.HEXAGON) {
 				idx = svg.indexOf("points=\"", idx + 1);
-				final List<Point2D.Double> points = SvekUtils.extractPointsList(svg, idx, fullHeight);
+				final int starting = idx;
+				final List<Point2D.Double> points = svgResult.substring(starting).extractList(SvgResult.POINTS_EQUALS);
 				final double minX = SvekUtils.getMinX(points);
 				final double minY = SvekUtils.getMinY(points);
-				corner1.manage(minX, minY);
-				sh.moveSvek(minX, minY);
-				sh.setOctagon(minX, minY, points);
-			} else if (sh.getType() == ShapeType.CIRCLE || sh.getType() == ShapeType.CIRCLE_IN_RECT
-					|| sh.getType() == ShapeType.OVAL) {
+				// corner1.manage(minX, minY);
+				node.moveSvek(minX, minY);
+				node.setPolygon(minX, minY, points);
+			} else if (node.getType() == ShapeType.CIRCLE || node.getType() == ShapeType.CIRCLE_IN_RECT
+					|| node.getType() == ShapeType.OVAL) {
 				final double cx = SvekUtils.getValue(svg, idx, "cx");
 				final double cy = SvekUtils.getValue(svg, idx, "cy") + fullHeight;
 				final double rx = SvekUtils.getValue(svg, idx, "rx");
 				final double ry = SvekUtils.getValue(svg, idx, "ry");
-				sh.moveSvek(cx - rx, cy - ry);
+				node.moveSvek(cx - rx, cy - ry);
 			} else {
-				throw new IllegalStateException(sh.getType().toString() + " " + sh.getUid());
+				throw new IllegalStateException(node.getType().toString() + " " + node.getUid());
 			}
 		}
 
 		for (Cluster cluster : bibliotekon.allCluster()) {
+			if (mergeIntricated) {
+				final IGroup group = cluster.getGroups().iterator().next();
+				if (entityFactory.isIntricated(group) != null) {
+					continue;
+				}
+			}
 			int idx = getClusterIndex(svg, cluster.getColor());
-			final List<Point2D.Double> points = SvekUtils.extractPointsList(svg, idx, fullHeight);
+			final int starting = idx;
+			final List<Point2D.Double> points = svgResult.substring(starting).extractList(SvgResult.POINTS_EQUALS);
 			final double minX = SvekUtils.getMinX(points);
 			final double minY = SvekUtils.getMinY(points);
 			final double maxX = SvekUtils.getMaxX(points);
 			final double maxY = SvekUtils.getMaxY(points);
 			cluster.setPosition(minX, minY, maxX, maxY);
-			corner1.manage(minX, minY);
+			// corner1.manage(minX, minY);
 
 			if (cluster.getTitleAndAttributeWidth() == 0 || cluster.getTitleAndAttributeHeight() == 0) {
 				continue;
 			}
 			idx = getClusterIndex(svg, cluster.getTitleColor());
-			final List<Point2D.Double> pointsTitle = SvekUtils.extractPointsList(svg, idx, fullHeight);
+			final int starting1 = idx;
+			final List<Point2D.Double> pointsTitle = svgResult.substring(starting1)
+					.extractList(SvgResult.POINTS_EQUALS);
 			final double minXtitle = SvekUtils.getMinX(pointsTitle);
 			final double minYtitle = SvekUtils.getMinY(pointsTitle);
 			cluster.setTitlePosition(minXtitle, minYtitle);
 		}
 
-		for (Line line : bibliotekon.allLines()) {
-			line.solveLine(svg, fullHeight, corner1);
+		for (SvekLine line : bibliotekon.allLines()) {
+			line.solveLine(svgResult);
 		}
 
-		for (Line line : bibliotekon.allLines()) {
-			line.manageCollision(bibliotekon.allShapes());
+		for (SvekLine line : bibliotekon.allLines()) {
+			line.manageCollision(bibliotekon.allNodes());
 		}
-		corner1.manage(0, 0);
-		return new ClusterPosition(corner1.getMinX(), corner1.getMinY(), fullWidth, fullHeight);
-		// return new ClusterPosition(0, 0, fullWidth, fullHeight);
+		// corner1.manage(0, 0);
+//		return new ClusterPosition(corner1.getMinX(), corner1.getMinY(), fullWidth, fullHeight);
+//		// return new ClusterPosition(0, 0, fullWidth, fullHeight);
 	}
 
 	private int getClusterIndex(final String svg, int colorInt) {
-		final String colorString = StringUtils.goLowerCase(StringUtils.getAsHtml(colorInt));
+		final String colorString = StringUtils.goLowerCase(DotStringFactory.sharp000000(colorInt));
 		final String keyTitle1 = "=\"" + colorString + "\"";
 		int idx = svg.indexOf(keyTitle1);
 		if (idx == -1) {
@@ -451,25 +470,32 @@ public class DotStringFactory implements Moveable {
 		return idx;
 	}
 
-	public void openCluster(IGroup g, int titleAndAttributeWidth, int titleAndAttributeHeight, TextBlock title,
-			TextBlock stereo) {
-		this.current = current.createChild(g, titleAndAttributeWidth, titleAndAttributeHeight, title, stereo,
-				colorSequence, skinParam);
+	public static String sharp000000(int color) {
+		final int v = 0xFFFFFF & color;
+		String s = "000000" + Integer.toHexString(v).toUpperCase();
+		s = s.substring(s.length() - 6);
+		return "#" + s;
+	}
+
+	public void openCluster(int titleAndAttributeWidth, int titleAndAttributeHeight, TextBlock title, TextBlock stereo,
+			IGroup g) {
+		this.current = current.createChild(titleAndAttributeWidth, titleAndAttributeHeight, title, stereo,
+				colorSequence, skinParam, g);
 		bibliotekon.addCluster(this.current);
 	}
 
 	public void closeCluster() {
-		if (current.getParent() == null) {
+		if (current.getParentCluster() == null) {
 			throw new IllegalStateException();
 		}
-		this.current = current.getParent();
+		this.current = current.getParentCluster();
 	}
 
 	public void moveSvek(double deltaX, double deltaY) {
-		for (Shape sh : bibliotekon.allShapes()) {
+		for (SvekNode sh : bibliotekon.allNodes()) {
 			sh.moveSvek(deltaX, deltaY);
 		}
-		for (Line line : bibliotekon.allLines()) {
+		for (SvekLine line : bibliotekon.allLines()) {
 			line.moveSvek(deltaX, deltaY);
 		}
 		for (Cluster cl : bibliotekon.allCluster()) {
