@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -38,6 +38,7 @@ package net.sourceforge.plantuml.svek;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
+import java.util.Set;
 
 import net.sourceforge.plantuml.ColorParam;
 import net.sourceforge.plantuml.FontParam;
@@ -53,19 +54,18 @@ import net.sourceforge.plantuml.cucadiagram.ILeaf;
 import net.sourceforge.plantuml.cucadiagram.LeafType;
 import net.sourceforge.plantuml.cucadiagram.Link;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
+import net.sourceforge.plantuml.cucadiagram.SuperGroup;
 import net.sourceforge.plantuml.cucadiagram.dot.DotData;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
-import net.sourceforge.plantuml.graphic.TextBlockEmpty;
-import net.sourceforge.plantuml.graphic.TextBlockWidth;
-import net.sourceforge.plantuml.graphic.TextBlockWidthAdapter;
 import net.sourceforge.plantuml.graphic.color.ColorType;
 import net.sourceforge.plantuml.skin.rose.Rose;
+import net.sourceforge.plantuml.style.SName;
 import net.sourceforge.plantuml.svek.image.EntityImageState;
 import net.sourceforge.plantuml.ugraphic.UStroke;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 public final class GroupPngMakerState {
 
@@ -75,6 +75,18 @@ public final class GroupPngMakerState {
 
 	class InnerGroupHierarchy implements GroupHierarchy {
 
+		public Set<SuperGroup> getAllSuperGroups() {
+			throw new UnsupportedOperationException();
+		}
+
+		public IGroup getRootGroup() {
+			throw new UnsupportedOperationException();
+		}
+
+		public SuperGroup getRootSuperGroup() {
+			throw new UnsupportedOperationException();
+		}
+
 		public Collection<IGroup> getChildrenGroups(IGroup parent) {
 			if (EntityUtils.groupRoot(parent)) {
 				return diagram.getChildrenGroups(group);
@@ -98,7 +110,7 @@ public final class GroupPngMakerState {
 	}
 
 	private List<Link> getPureInnerLinks() {
-		final List<Link> result = new ArrayList<Link>();
+		final List<Link> result = new ArrayList<>();
 		for (Link link : diagram.getLinks()) {
 			if (EntityUtils.isPureInnerLink12(group, link)) {
 				result.add(link);
@@ -110,9 +122,8 @@ public final class GroupPngMakerState {
 	public IEntityImage getImage() {
 		final Display display = group.getDisplay();
 		final ISkinParam skinParam = diagram.getSkinParam();
-		final TextBlock title = display.create(
-				new FontConfiguration(skinParam, FontParam.STATE, group.getStereotype()), HorizontalAlignment.CENTER,
-				diagram.getSkinParam());
+		final TextBlock title = display.create(new FontConfiguration(skinParam, FontParam.STATE, group.getStereotype()),
+				HorizontalAlignment.CENTER, diagram.getSkinParam());
 
 		if (group.size() == 0 && group.getChildren().size() == 0) {
 			return new EntityImageState(group, diagram.getSkinParam());
@@ -124,8 +135,8 @@ public final class GroupPngMakerState {
 				diagram.isHideEmptyDescriptionForState(), DotMode.NORMAL, diagram.getNamespaceSeparator(),
 				diagram.getPragma());
 
-		final DotDataImageBuilder svek2 = new DotDataImageBuilder(dotData, diagram.getEntityFactory(),
-				diagram.getSource(), diagram.getPragma(), stringBounder);
+		final GeneralImageBuilder svek2 = new GeneralImageBuilder(false, dotData, diagram.getEntityFactory(),
+				diagram.getSource(), diagram.getPragma(), stringBounder, SName.stateDiagram);
 
 		if (group.getGroupType() == GroupType.CONCURRENT_STATE) {
 			// return new InnerStateConcurrent(svek2.createFile());
@@ -136,45 +147,33 @@ public final class GroupPngMakerState {
 			throw new UnsupportedOperationException(group.getGroupType().toString());
 		}
 
-		HtmlColor borderColor = group.getColors(skinParam).getColor(ColorType.LINE);
+		HColor borderColor = group.getColors(skinParam).getColor(ColorType.LINE);
 		if (borderColor == null) {
 			borderColor = getColor(ColorParam.stateBorder, group.getStereotype());
 		}
 		final Stereotype stereo = group.getStereotype();
-		final HtmlColor backColor = group.getColors(skinParam).getColor(ColorType.BACK) == null ? getColor(
-				ColorParam.stateBackground, stereo) : group.getColors(skinParam).getColor(ColorType.BACK);
-		final TextBlockWidth attribute = getAttributes(skinParam);
+		final HColor backColor = group.getColors(skinParam).getColor(ColorType.BACK) == null
+				? getColor(ColorParam.stateBackground, stereo)
+				: group.getColors(skinParam).getColor(ColorType.BACK);
+		final TextBlock attribute = GeneralImageBuilder.stateHeader((IEntity) group, null, skinParam);
 
 		final Stereotype stereotype = group.getStereotype();
 		final boolean withSymbol = stereotype != null && stereotype.isWithOOSymbol();
 
 		final boolean containsOnlyConcurrentStates = containsOnlyConcurrentStates(dotData);
-		final IEntityImage image = containsOnlyConcurrentStates ? buildImageForConcurrentState(dotData) : svek2
-				.buildImage(null, new String[0]);
+		final IEntityImage image = containsOnlyConcurrentStates ? buildImageForConcurrentState(dotData)
+				: svek2.buildImage(null, new String[0]);
 		UStroke stroke = group.getColors(skinParam).getSpecificLineStroke();
 		if (stroke == null) {
 			stroke = new UStroke(1.5);
 		}
-		return new InnerStateAutonom(image, title, attribute, borderColor, backColor, skinParam.shadowing(),
-				group.getUrl99(), withSymbol, stroke);
-
-	}
-
-	private TextBlockWidth getAttributes(final ISkinParam skinParam) {
-		final List<String> details = ((IEntity) group).getBodier().getRawBody();
-
-		if (details.size() == 0) {
-			return new TextBlockEmpty();
-		}
-		final FontConfiguration fontConfiguration = new FontConfiguration(skinParam, FontParam.STATE_ATTRIBUTE, null);
-		final Display display = details.size() == 1 ? Display.getWithNewlines(details.get(0)) : Display.create(details);
-		final TextBlock result = display.create(fontConfiguration, HorizontalAlignment.LEFT, skinParam);
-		return new TextBlockWidthAdapter(result, 0);
+		return new InnerStateAutonom(image, title, attribute, borderColor, backColor,
+				skinParam.shadowing(group.getStereotype()), group.getUrl99(), withSymbol, stroke);
 
 	}
 
 	private IEntityImage buildImageForConcurrentState(DotData dotData) {
-		final List<IEntityImage> inners = new ArrayList<IEntityImage>();
+		final List<IEntityImage> inners = new ArrayList<>();
 		for (ILeaf inner : dotData.getLeafs()) {
 			inners.add(inner.getSvekImage());
 		}
@@ -197,8 +196,8 @@ public final class GroupPngMakerState {
 
 	private final Rose rose = new Rose();
 
-	private HtmlColor getColor(ColorParam colorParam, Stereotype stereo) {
+	private HColor getColor(ColorParam colorParam, Stereotype stereo) {
 		final ISkinParam skinParam = diagram.getSkinParam();
-		return rose.getHtmlColor(skinParam, colorParam, stereo);
+		return rose.getHtmlColor(skinParam, stereo, colorParam);
 	}
 }
