@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -37,18 +37,19 @@ package net.sourceforge.plantuml.syntax;
 
 import java.io.PrintStream;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.Set;
 import java.util.TreeSet;
 
 import net.sourceforge.plantuml.SkinParam;
-import net.sourceforge.plantuml.graphic.HtmlColorSetSimple;
+import net.sourceforge.plantuml.ugraphic.color.HColorSet;
 import net.sourceforge.plantuml.utils.Cypher;
 
 public class LanguageDescriptor {
 
-	private final Set<String> type = new TreeSet<String>();
-	private final Set<String> keyword = new TreeSet<String>();
-	private final Set<String> preproc = new TreeSet<String>();
+	private final Set<String> type = new TreeSet<>();
+	private final Set<String> keyword = new TreeSet<>();
+	private final Set<String> preproc = new TreeSet<>();
 
 	public LanguageDescriptor() {
 
@@ -80,13 +81,60 @@ public class LanguageDescriptor {
 		type.add("package");
 		type.add("queue");
 		type.add("archimate");
+		type.add("diamond");
+		type.add("detach");
+		type.add("collections");
+		type.add("hexagon");
+		type.add("label");
+		type.add("person");
+		type.add("annotation");
 
+		keyword.add("@startwire");
+		keyword.add("@startbpm");
 		keyword.add("@startuml");
-		keyword.add("@enduml");
 		keyword.add("@startdot");
-		keyword.add("@enddot");
+		keyword.add("@startjcckit");
+		keyword.add("@startditaa");
+		keyword.add("@startproject");
 		keyword.add("@startsalt");
+		keyword.add("@startflow");
+		keyword.add("@startcreole");
+		keyword.add("@starttree");
+		keyword.add("@startcute");
+		keyword.add("@startmath");
+		keyword.add("@startlatex");
+		keyword.add("@startdef");
+		keyword.add("@startgantt");
+		keyword.add("@startnwdiag");
+		keyword.add("@startmindmap");
+		keyword.add("@startwbs");
+		keyword.add("@startjson");
+		keyword.add("@startgit");
+		keyword.add("@startboard");
+		keyword.add("@startyaml");
+		keyword.add("@endwire");
+		keyword.add("@endbpm");
+		keyword.add("@enduml");
+		keyword.add("@enddot");
+		keyword.add("@endjcckit");
+		keyword.add("@endditaa");
+		keyword.add("@endproject");
 		keyword.add("@endsalt");
+		keyword.add("@endflow");
+		keyword.add("@endcreole");
+		keyword.add("@endtree");
+		keyword.add("@endcute");
+		keyword.add("@endmath");
+		keyword.add("@endlatex");
+		keyword.add("@enddef");
+		keyword.add("@endgantt");
+		keyword.add("@endnwdiag");
+		keyword.add("@endmindmap");
+		keyword.add("@endwbs");
+		keyword.add("@endjson");
+		keyword.add("@endgit");
+		keyword.add("@endboard");
+		keyword.add("@endyaml");
 		keyword.add("as");
 		keyword.add("also");
 		keyword.add("autonumber");
@@ -125,12 +173,14 @@ public class LanguageDescriptor {
 		keyword.add("top");
 		keyword.add("bottom");
 		keyword.add("top to bottom direction");
+		keyword.add("left to right direction");
 		keyword.add("package");
 		keyword.add("namespace");
 		keyword.add("page");
 		keyword.add("up");
 		keyword.add("down");
 		keyword.add("if");
+		keyword.add("then");
 		keyword.add("else");
 		keyword.add("elseif");
 		keyword.add("endif");
@@ -151,9 +201,34 @@ public class LanguageDescriptor {
 		keyword.add("again");
 		keyword.add("kill");
 		keyword.add("order");
+		keyword.add("allow_mixing");
+		keyword.add("allowmixing");
+		keyword.add("mainframe");
+		keyword.add("across");
+		keyword.add("stereotype");
+		keyword.add("split");
+		keyword.add("style");
+		keyword.add("sprite");		
+		keyword.add("circle");
+		keyword.add("empty");
+		keyword.add("members");
+		keyword.add("description");
+		keyword.add("true");
+		keyword.add("false");
+		keyword.add("normal");
+		keyword.add("italic");
+		keyword.add("bold");
+		keyword.add("plain");
+		keyword.add("color");
+		keyword.add("dotted");
+		keyword.add("dashed");
+		keyword.add("bold");
+		keyword.add("map");
+
 
 		preproc.add("!exit");
 		preproc.add("!include");
+		preproc.add("!theme");
 		preproc.add("!pragma");
 		preproc.add("!define");
 		preproc.add("!undef");
@@ -164,6 +239,19 @@ public class LanguageDescriptor {
 		preproc.add("!else");
 		preproc.add("!definelong");
 		preproc.add("!enddefinelong");
+		preproc.add("!function");
+		preproc.add("!procedure");
+		preproc.add("!endfunction");
+		preproc.add("!endprocedure");
+		preproc.add("!unquoted");
+		preproc.add("!return");
+		preproc.add("!startsub");
+		preproc.add("!endsub");
+		preproc.add("!assert");
+		preproc.add("!log");
+		preproc.add("!local");
+		preproc.add("!dump_memory");
+		preproc.add("!import");
 	}
 
 	public Cypher getCypher() {
@@ -180,9 +268,10 @@ public class LanguageDescriptor {
 		for (String s : SkinParam.getPossibleValues()) {
 			cypher.addException(s);
 		}
-		for (String s : new HtmlColorSetSimple().names()) {
+		for (String s : HColorSet.instance().names()) {
 			cypher.addException(s);
 		}
+		cypher.addException("o");
 		return cypher;
 	}
 
@@ -191,7 +280,7 @@ public class LanguageDescriptor {
 		print(ps, "keyword", keyword);
 		print(ps, "preprocessor", preproc);
 		print(ps, "skinparameter", SkinParam.getPossibleValues());
-		print(ps, "color", new HtmlColorSetSimple().names());
+		print(ps, "color", HColorSet.instance().names());
 		ps.println(";EOF");
 	}
 
@@ -204,4 +293,16 @@ public class LanguageDescriptor {
 		ps.println();
 	}
 
+	public final Set<String> getType() {
+		return Collections.unmodifiableSet(type);
+	}
+
+	public final Set<String> getKeyword() {
+		return Collections.unmodifiableSet(keyword);
+	}
+
+	public final Set<String> getPreproc() {
+		return Collections.unmodifiableSet(preproc);
+	}
+
 }
