@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -34,34 +34,46 @@
  */
 package net.sourceforge.plantuml.timingdiagram;
 
-import java.awt.Font;
 import java.awt.geom.Dimension2D;
 import java.awt.geom.Point2D;
+import java.util.List;
+import java.util.Objects;
 
 import net.sourceforge.plantuml.ISkinParam;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColorUtils;
+import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
-import net.sourceforge.plantuml.ugraphic.UChangeBackColor;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
+import net.sourceforge.plantuml.style.PName;
+import net.sourceforge.plantuml.style.SName;
+import net.sourceforge.plantuml.style.Style;
+import net.sourceforge.plantuml.style.StyleBuilder;
+import net.sourceforge.plantuml.style.StyleSignature;
+import net.sourceforge.plantuml.timingdiagram.graphic.TimeArrow;
 import net.sourceforge.plantuml.ugraphic.UFont;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.ULine;
 import net.sourceforge.plantuml.ugraphic.UPolygon;
+import net.sourceforge.plantuml.ugraphic.UStroke;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
+import net.sourceforge.plantuml.ugraphic.color.HColorUtils;
 
 public class TimeConstraint {
 
 	private final TimeTick tick1;
 	private final TimeTick tick2;
 	private final Display label;
+	private final ISkinParam skinParam;
+	private final StyleBuilder styleBuilder;
 
-	public TimeConstraint(TimeTick tick1, TimeTick tick2, String label) {
-		this.tick1 = tick1;
-		this.tick2 = tick2;
+	public TimeConstraint(TimeTick tick1, TimeTick tick2, String label, ISkinParam skinParam) {
+		this.tick1 = Objects.requireNonNull(tick1);
+		this.tick2 = Objects.requireNonNull(tick2);
 		this.label = Display.getWithNewlines(label);
+		this.skinParam = skinParam;
+		this.styleBuilder = skinParam.getCurrentStyleBuilder();
 	}
 
 	public final TimeTick getTick1() {
@@ -76,33 +88,63 @@ public class TimeConstraint {
 		return label;
 	}
 
-	private TextBlock getTextBlock(Display display, ISkinParam skinParam) {
+	private TextBlock getTextBlock(Display display) {
 		return display.create(getFontConfiguration(), HorizontalAlignment.LEFT, skinParam);
 	}
 
 	private FontConfiguration getFontConfiguration() {
 		final UFont font = UFont.serif(14);
-		return new FontConfiguration(font, HtmlColorUtils.BLACK, HtmlColorUtils.BLUE, false);
+		return new FontConfiguration(font, HColorUtils.BLACK, HColorUtils.BLUE, false);
 	}
 
-	public void drawU(UGraphic ug, TimingRuler ruler, ISkinParam skinParam) {
-		ug = ug.apply(new UChangeColor(HtmlColorUtils.RED)).apply(new UChangeBackColor(HtmlColorUtils.RED));
+	public void drawU(UGraphic ug, TimingRuler ruler) {
+		final HColor arrowColor = getArrowColor();
+		ug = ug.apply(arrowColor).apply(arrowColor.bg());
 		final double x1 = ruler.getPosInPixel(tick1);
 		final double x2 = ruler.getPosInPixel(tick2);
-		final ULine line = new ULine(x2 - x1, 0);
-
-		ug = ug.apply(new UTranslate(x1, 0));
-		ug.draw(line);
+		ug = ug.apply(UTranslate.dx(x1));
+		ug.apply(getUStroke()).draw(ULine.hline(x2 - x1));
 
 		ug.draw(getPolygon(-Math.PI / 2, new Point2D.Double(0, 0)));
 		ug.draw(getPolygon(Math.PI / 2, new Point2D.Double(x2 - x1, 0)));
 
-		final TextBlock text = getTextBlock(label, skinParam);
+		final TextBlock text = getTextBlock(label);
 		final Dimension2D dimText = text.calculateDimension(ug.getStringBounder());
 		final double x = (x2 - x1 - dimText.getWidth()) / 2;
-		final UTranslate tr = new UTranslate(x, -5 - dimText.getHeight());
-		text.drawU(ug.apply(tr));
+		text.drawU(ug.apply(new UTranslate(x, -getConstraintHeight(ug.getStringBounder()))));
+	}
 
+	private HColor getArrowColor() {
+		if (styleBuilder == null) {
+			return HColorUtils.MY_RED;
+		}
+		return getStyle().value(PName.LineColor).asColor(skinParam.getThemeStyle(), skinParam.getIHtmlColorSet());
+	}
+
+	private Style getStyle() {
+		return getStyleSignature().getMergedStyle(styleBuilder);
+	}
+
+	private UStroke getUStroke() {
+		if (styleBuilder == null) {
+			return new UStroke(1.5);
+		}
+		return getStyle().getStroke();
+	}
+
+	private StyleSignature getStyleSignature() {
+		return StyleSignature.of(SName.root, SName.element, SName.timingDiagram, SName.constraintArrow);
+	}
+
+	public double getConstraintHeight(StringBounder stringBounder) {
+		final TextBlock text = getTextBlock(label);
+		final Dimension2D dimText = text.calculateDimension(stringBounder);
+		return dimText.getHeight() + getTopMargin();
+
+	}
+
+	public static double getTopMargin() {
+		return 5;
 	}
 
 	private UPolygon getPolygon(final double angle, final Point2D end) {
@@ -118,4 +160,16 @@ public class TimeConstraint {
 		return polygon;
 	}
 
+	public static double getHeightForConstraints(StringBounder stringBounder, List<TimeConstraint> constraints) {
+		if (constraints.size() == 0) {
+			return 0;
+		}
+		double result = 0;
+		for (TimeConstraint constraint : constraints) {
+			result = Math.max(result, constraint.getConstraintHeight(stringBounder));
+
+		}
+		return result;
+	}
+
 }
