@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -39,8 +39,8 @@ import java.awt.geom.Dimension2D;
 import java.awt.geom.Point2D;
 
 import net.sourceforge.plantuml.Dimension2DDouble;
-import net.sourceforge.plantuml.graphic.HtmlColor;
-import net.sourceforge.plantuml.graphic.HtmlColorUtils;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
+import net.sourceforge.plantuml.ugraphic.color.HColorUtils;
 
 public class MinMax {
 
@@ -90,6 +90,18 @@ public class MinMax {
 		this.minY = minY;
 		this.maxX = maxX;
 		this.maxY = maxY;
+		if (Double.isNaN(minX)) {
+			throw new IllegalArgumentException();
+		}
+		if (Double.isNaN(maxX)) {
+			throw new IllegalArgumentException();
+		}
+		if (Double.isNaN(minY)) {
+			throw new IllegalArgumentException();
+		}
+		if (Double.isNaN(maxY)) {
+			throw new IllegalArgumentException();
+		}
 	}
 
 	public MinMax addPoint(Point2D pt) {
@@ -142,8 +154,11 @@ public class MinMax {
 	}
 
 	public void drawGrey(UGraphic ug) {
-		final HtmlColor color = HtmlColorUtils.GRAY;
-		ug = ug.apply(new UChangeColor(color)).apply(new UChangeBackColor(color));
+		draw(ug, HColorUtils.GRAY);
+	}
+
+	public void draw(UGraphic ug, HColor color) {
+		ug = ug.apply(color).apply(color.bg());
 		ug = ug.apply(new UTranslate(minX, minY));
 		ug.draw(new URectangle(getWidth(), getHeight()));
 	}
