@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -35,15 +35,16 @@
  */
 package net.sourceforge.plantuml.version;
 
-import java.net.URL;
 import java.util.Date;
 
+import net.sourceforge.plantuml.security.SURL;
+
 public class Version {
 
 	private static final int MAJOR_SEPARATOR = 1000000;
 
 	public static int version() {
-		return 1201809;
+		return 1202110;
 	}
 
 	public static int versionPatched() {
@@ -60,10 +61,14 @@ public class Version {
 		return dotted(version());
 	}
 
+	public static String fullDescription() {
+		return "PlantUML version " + Version.versionString() + " (" + Version.compileTimeString() + ")";
+	}
+
 	private static String dotted(int nb) {
 		final String minor = "" + nb % MAJOR_SEPARATOR;
 		final String major = "" + nb / MAJOR_SEPARATOR;
-		return major + "." + minor.substring(0, 4) + "." + minor.substring(4);
+		return major + "." + minor.substring(0, 4) + "." + Integer.parseInt(minor.substring(4));
 	}
 
 	public static String versionString(int size) {
@@ -88,7 +93,7 @@ public class Version {
 	}
 
 	public static long compileTime() {
-		return 1532710698423L;
+		return 1630331028954L;
 	}
 
 	public static String compileTimeString() {
@@ -104,7 +109,7 @@ public class Version {
 			if (loader == null) {
 				return "No ClassLoader?";
 			}
-			final URL url = loader.getResource("net/sourceforge/plantuml/version/Version.class");
+			final SURL url = SURL.create(loader.getResource("net/sourceforge/plantuml/version/Version.class"));
 			if (url == null) {
 				return "No URL?";
 			}
